package bibalign;

import java.io.*;
import java.util.regex.*;

/**Represents the row alignment
 *
 * @author Daniela Buchwald
 */
public class Alignment {
    private char[][] characters; //Saves all sequences
    private String[] seq_names; //Saves the names of the sequences
    private int num_of_nucleotides = 0; //Gets the number of nucleotides (in the wildtypesequence)  in the file 
    //without the gaps as an integer
    private static String file = null; //Will get the file
    private static Boolean is_legal = false;
    
    /**Constructs alignment from file
     * - Reads from the file_name
     * - Splits the single entries
     * - Gets the sequence names and saves them
     * - Gets the first line which is needed for the next step
     * - Counts the gaps in front and at the end of the first sequence
     * - Reads how many nucleotides shall be examined
     * - Saves the single nucleotide-characters 
     * 
     * @param file_name The filename of the file from which the 
     * alignment should be constructed
     * @throws  IllegalArgumentException
     */
    public Alignment(String filename, Boolean use_of_mut_sides)
            throws IllegalArgumentException
    {

        readFile(filename); //Read the file
	            
        if(is_legal) { //Function readFile returns a Boolean whether the file was read correct - this is checked here
            /*Spliting the single entrys*/
            final String NAMESTART_SYMBOL = ">"; /*The greater as symbol is in front of each 
                     * name so substring can find the name using this*/
            final String WHITESPACE = " "; //The first whitespace in the file is at the end of the name
            String []entrys = file.split(NAMESTART_SYMBOL); //The arrayfield which will contain the entrys which where splited at the >
        
            /*Gets the names of the sequences and puts them in the array seq_names*/
            int number_of_entrys = entrys.length; /*Each field of the array contains 
            * one entry so the number of fields == number of entrys*/
            //TODO: Container???
            int wildtype_seq_position = use_of_mut_sides ? 0 : 1;
            if(number_of_entrys < wildtype_seq_position+2) //Less than 2/3 (if no/there is mut_side_seq) sequences/entries are not legal as you need at least a wildtype and another for comparison
            {
                    throw new IllegalArgumentException("Found too less sequences");
            }
        
            number_of_entrys--; /* split() fabricates a entry which only contains a newline. 
            *This one isn't used but is count so the arraybounds will set wrong without the -1*/
            seq_names = new String[number_of_entrys]; //the field (seq_names) which gets the names 
            for(int i=1; i<=number_of_entrys;i++) //The entry[0] dosn't exist so the counter starts at 1
            {
                int start = entrys[i].indexOf(NAMESTART_SYMBOL); //Get the start of the name which contains ">"
                //TODO: ERROR: No namestart_symbol found
                Matcher matcher = Pattern.compile("\\s").matcher(entrys[i]);
                if(!matcher.find()) //If there was no whitespace for the end of entry found or an unlegal number comes up
                {
                    throw new IllegalArgumentException("Found no entry-end");
                }
                int end = matcher.start(); //Get the end of the name which is at the first whitespace
                seq_names[i-1] = entrys[i].substring(start+1, end); //Saves the name of the sequence
            }
        
            final String EMPTY = ""; 
            String entry_one = entrys[1].replaceAll(WHITESPACE, EMPTY); //Deletes all whitespaces because substring gets confused by them
        
            /*Gets the first line of an entry because this is needed for the Matcher*/        
            int first_line_length = entry_one.indexOf(System.getProperty("line.separator")); /*Gets the length of the first line from > to the first newline. 
            * Used for the calculation of the number of nucleotides and to get the first line.*/
            String first_line = entry_one.substring(0, first_line_length); //Gets the whole first line
        
            entry_one = entry_one.replaceAll(System.getProperty("line.separator"), EMPTY); //Deletes all newline because substring gets confused by them
  
            /*Counts the gaps in front and at the end of
            * the first sequence so they can be deleted in the other sequences
            * and in the end only the needed nucleotides are in the array
            */
            char searched_char = '-'; /**The char which is searched. In this case the gap-char "-" */
        
            Pattern p = Pattern.compile(first_line+"("+searched_char+"+)"+".+?"); //Defines the pattern which shall be searched
            Matcher m = p.matcher(entry_one); //Defines in which string will be searched
            int gaps_front=0, gaps_end=0; //Sets the gaps in front/end to zero so there is a value if there are no gaps
            int firstline = first_line_length; //If there are gaps in the front first_line_length needs an other value so there is a new variable for this
            Boolean first_find = m.find();
            if(first_find) //Search und edit gaps if something is found
            {    
                gaps_front = m.group(1).length()-1; //Contains the number of gaps in front of the sequence
                firstline = first_line_length+1;
            }
        
            Pattern pat = Pattern.compile(first_line+searched_char+"*?"+"[A-Z]"+"+?"+".+?"+"("+searched_char+"+)"); //defines the pattern which shall be searched
            Matcher mat = pat.matcher(entry_one); //Defines in which string will be searched
            if(mat.find()) //Search und edit gaps if something is found
            {    
                gaps_end = mat.group(1).length(); //Contains the number of gabs behind the sequence;
            }
        
            /**Calculates, how many nucelotides shall be examined*/
            num_of_nucleotides = entry_one.length() - gaps_front - gaps_end - firstline;
                //Length of the first entry minus gaps in front and at the end of the sequence minus the headeline
         
            /**Puts the sequences in the array "characters"*/
        
            characters = new char[number_of_entrys][num_of_nucleotides]; //Will get all entrys
        
            for(int j=1;j <= number_of_entrys;j++) //Goes through all entries
            {
                int k = 0; //Loop-counter, reset to zero
                first_line_length = entrys[j].indexOf(System.getProperty("line.separator")); //Find the first newline/carriage return
                if(first_find)
                    first_line_length += 1; //If gaps in front of the sequence were founded add a one for correct calculation
                String entry_j = entrys[j].replaceAll(System.getProperty("line.separator"), EMPTY); //Remove all newlines/carriage returns
                int begin = first_line_length + gaps_front; //Where the sequence should start
                int finish = entrys[j].length() - gaps_end; //Where the sequence should stop
            
                while(begin < finish && k < num_of_nucleotides) //While the begin is before the end and k isn't bigger than number of nucleotides
                //TODO: 2 counters needed?
                {
                    char nucleotid = entry_j.charAt(begin); //Read the current nucleotide from entry
                    ++begin; //go to next char
                    characters[j-1][k] = nucleotid; //Save the read char
                    ++k;
                }
            }
        }
    }
    
    /**Gets character at given coordinate
     * 
     * @param no_seq The number of the sequence
     * @param pos The position where to get the char
     * @return The character at given coordinate
     * @throws IllegalArgumentException
     */
    public char get_character(int no_seq, int pos)
            throws IllegalArgumentException
    {
        int num_all_sequences =  number_of_all_sequences(); //Get how much sequences were in the inputfile
        
        if(pos >= num_of_nucleotides || no_seq >= num_all_sequences) //Check whether
        {
                throw new IllegalArgumentException("Char from sequence at given coordinate not found");
        }
        return characters[no_seq][pos]; //Return the character at the given coordinate in the given sequence
    }
    
    /**Gets the name of a sequence.
     * The sequence which the method will get is defined through the given number no
     * 
     * @param no The number of the sequence from which the name is wanted
     * @return The Name of the Sequence
     * @throws IllegalArgumentException
     */
    public String get_seq_name(int no)
            throws IllegalArgumentException
    {
        if(no > seq_names.length) //When the given coordinate are illegal
        {
                throw new IllegalArgumentException("Sequencename not found" 
                        + "(Number of sequence that should be returend greater then number of sequences in the alignment");
        }
        return seq_names[no]; //Return the name of the requested sequence
    }
    
    /**Returns the number of entrys
     * 
     * @return How much entrys has the alignment
     */
    public int number_of_all_sequences()
    {
        return seq_names.length; //Number of sequences in the alignment
    }
    
    /**Returns the number of nucleotides
     * 
     * @return Number of nucleotides
     */
    public int number_of_nucleotides()
    {
        return num_of_nucleotides; //The number of nucleotides in the sequences
    }
    
    /**Reads the file to build a alignment from it
     * 
     * @param filename The file for the alignment
     */
    public static void readFile(String filename) {
                /* Reading from the file*/
        StringBuffer input = new StringBuffer();
	String line = null; //Will get a single line of the file
        is_legal = true; //build was sucessfully
        try {
            BufferedReader buffreader = new BufferedReader(new FileReader(filename));
            while ((line = buffreader.readLine()) != null) //read
            file = (input.append(System.getProperty("line.separator")).append(line)).toString(); //and put it in the input

        } catch (FileNotFoundException ex) {
            GUI.addWarningMessage("File " + filename +" not found");
            is_legal = false; //an exception occured
        } catch (IOException ex) {
            GUI.addWarningMessage("Unknown IOException occured: " + ex);
            is_legal = false; //an exception occured
        }
    }
    
    /**Returns whether the making of the alignment was complete or not
     * 
     * @return Boolean true - build was sucessfully, false - an exception occured
     */
    public Boolean isLegal() {
        return is_legal;
    }
}
