package bibalign;

/** This class contains functions for the used alphabet
 *
 * @author Daniela buchwald
 */
public class Alphabet {
    
    private static boolean is_init; //Whether the alphabet is intialized
    private static char[] legal_characters; //All the legal characters in the used alphabet
    private static String[] groups; //The matching groups for the used characters
    
    /**Define which characters are considered to be legal 
     * (A, C, G, T, - in our case), and to which group the characters 
     * belong to (pyrimide/purine in our case) 
     * 
     * @param given_characters The characters which are legal
     * @param group The group of the nucleotide
     */
    public static void init(char[] given_characters, String[] given_groups)
    {
        legal_characters = given_characters; //Get all legal characters
        groups = given_groups; //Get the matching group to the characters
        is_init = true; //Notice that the alphabet is initialized
    }
    
    /**Is the character different but still in the same group?
     * 
     * @param first_char The char to compare with
     * @param second_char The char which is compared to the first char
     * @return If the character is different and in the same group (true/false)
     * @throws Exception
     */
    public static boolean is_same_group_change(char first_char, char second_char)
            throws Exception
    {       
        if(!is_init) //If the alphabet wasn't initialized yet
        {
                throw new Exception("Legal chars not initialized"); 
        }
        int first_group_number = 0, second_group_number = 0; //Gets the groupnumbers for comparation
        
        if(first_char != second_char) //If the chars differ
        {
            for(int i=0; i < groups.length; i++)
            {
                if(first_char == legal_characters[i]) //If the first char is the same than that one on the list at the i-th position save i 
                    first_group_number = i; //Gets the first groupnumber for comparation
                if(second_char == legal_characters[i]) //If the second char meets the char at the i-th position in the list store i
                    second_group_number = i; //Gets the second groupnumber for comparation
            }
        
            if(groups[first_group_number].equals(groups[second_group_number])) //If the groups of the chars are still the same
                return true;
            else //If the groups aren't the samt return false
                return false;
         }
         else //If the char is the same, return false
             return false;
    }
    
    /**Is the character different and in an other group?
     * 
     * Gaps has to be catched before entering the method.
     * 
     * @param first_char The char to compare with
     * @param second_char The char which is compared to the first char
     * @return If the character is different and in a different group (true/false)
     * @throws Exception
     */
    public static boolean is_different_group_change(char first_char, char second_char)
            throws Exception
    {
        if(!is_init) //If the alphabet wasn't initialized yet
        {
                throw new Exception("Legal chars not initialized");
        }
        int first_group_number = 0, second_group_number = 0; //Gets the groupnumbers for comparation
        
            if(first_char != second_char) //If the chars differ
            {
                int array_length = groups.length; //Get how much groups are given
                for(int i=0; i < array_length; i++)
                {
                    if(first_char == legal_characters[i])
                    first_group_number = i; //Gets the first groupnumber for comparation
                    if(second_char == legal_characters[i])
                    second_group_number = i; //Gets the second groupnumber for comparation
                }
                    //TODO: Hilfsfunktion 1?
        
                if(groups[first_group_number].equals(groups[second_group_number]))
                      return false;
                else //If the groups aren't the samt return false
                    return true;
            }
            else //If the char is the same, return false
                return false;
    }
    
    /**This function returns the legal characters
     * 
     * @return The legal characters
     * @throws Exception
     */
    public static char[] return_legal_characters()
            throws Exception
    {
        if(!is_init) //If the alphabet wasn't initialized yet
        {
                throw new Exception("Legal chars not initialized");
        }
        return legal_characters; //Return all legal characters
    }

}
