package bibalign;

import com.mallardsoft.tuple.*;

/**This class represents an amino acid
 * 
 * @author Daniela Buchwald
 *
 */
public class Amino_Acid { 

	private char amino_acid; //The char of the amino acid (if invalid: X)
	private boolean is_legal; //If the amino acid is legal
        private static Used_Amino_Acids used_aminos = null; //Used to check whether the map is initiated or not 

	/**This function constructs a amino acid from a given triplet
         *
	 * @param triplet The triplet from which the 
         * Amino_Acid should be constructed
	 */
	public Amino_Acid(Triplet triplet)
	{         
            Pair<Character, Boolean> amino_return = used_aminos.convert(triplet); /*Gets the character and 
             * if the amino acid is legal which is coded by the given triplet */
            
            amino_acid = Pair.get1(amino_return); //Sets the character for the amino
            is_legal = Pair.get2(amino_return); //Sets if the amino is legal
	}
        
        /**Constructor for an amino acid.
         * Should be used for the initiation of the map.
         * Can only be used if the map (Used_Amino_Acids-object) isn't initiated yet which means
         * that the private variable used_aminos is null
         * or when the character is X.
         * 
         * @param character the character of the new amino acid
         * @throws Exception
         */
        public Amino_Acid(char character)
                throws Exception
        {
            if(used_aminos == null || character == 'X') //If the map isn't initiated yet OR if the given character is a X
            {
               amino_acid = character; //Sets the charactor for the amino
               is_legal = true; /*As the constructor gets the character 
               * from the Used_Amino_Acids-method, it has to be legal*/
            }
            else
                    throw new Exception("Used_Aminos is already initialized");
        }
	
        /**Initializes the used_aminos-list
         * 
         * @param given_used_aminos
         */
        static void init(Used_Amino_Acids given_used_aminos) 
        {
            used_aminos = given_used_aminos; //Copies the given map to the local one
        }
        
	/**Convert amino acid to char
	 * 
	 *@return The Char of the Amino.
	 */
	public char to_char()
	{
            char amino = this.amino_acid;
            return amino;
	}
        
        /**Checks if the amino acid is legal
         * 
         * @return If the amino acid is legal
         */
        public Boolean is_legal()
        {
            return is_legal;
        }
}
