package bibalign;

import jxl.write.*;
import jxl.*;
import java.io.*;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;

/**Grafic User Interface for the programm
 *
 * @author Daniela Buchwald
 */
public class GUI extends JFrame{
    
    private String excelname = "output"; //Gets the name of the excelfile which will be created
    private String filename = "testdata/test_alignment.fas"; //Gets the name of the file containing the alignment
    private String aminotablename = "all_aminos.xls"; //Gets the name of the amino acids file
    private String singledeletion = "-"; //Gets the representation of a single deletion
    private String doubledeletion = "--"; //Gets the representation of a double deletion
    private String trimerdeletion = "---"; //Gets the representation of a trimer deletion
    private static String warningmessages = "";
    private static final TextArea warnings = new TextArea( " ",5, 10000 );


    
    /**Constructor for the GUI
     * 
     */
    public GUI()
    { 
    this.setTitle( "MuSi - Mutation Sites" );
    this.setSize( 1024,768);
    
    final JPanel panel = new JPanel(); 
    getContentPane().add(panel);
    panel.setLayout(null);
    
    try {
        String inputNames[] = readConfigFile();
        filename = inputNames[1];
        excelname = inputNames[2];
        aminotablename = inputNames[3];
        singledeletion = inputNames[4];
        doubledeletion = inputNames[5];
        trimerdeletion = inputNames[6];
    } catch(NullPointerException ex) {

    }

    
    //All components for the input data    
    final JLabel inputlabel = new JLabel("Input data:"); 
    inputlabel.setBounds(15, 10, 160, 30);
    
    final JPanel input_data_panel = new JPanel(); 
    input_data_panel.setBounds(15, 35, 490, 260);
    input_data_panel.setBorder(BorderFactory.createLineBorder(Color.black));
    input_data_panel.setLayout(null);
    
    final JLabel excellabel = new JLabel("Output file: "); //Create a label to identify which textfield is for the excelfilename
    excellabel.setBounds(10, 60, 150, 30);
    final JTextField excelnamefield = new JTextField( excelname, 40 ); //Create a textfield to write in the excelfilename
    excelnamefield.setBounds(170, 60, 150, 30);
    excelnamefield.setToolTipText("Insert the name the excelfile will have"); //Tooltip for the excelfile-textfield
    excelnamefield.addActionListener( new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
        }
    } ); 
            final JFileChooser savechooser = new JFileChooser();
    savechooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    final JButton saveButtonFile = new JButton("Save as...");
    saveButtonFile.addActionListener(new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
            savechooser.showOpenDialog(panel);
            File file = savechooser.getSelectedFile();
            excelname = file.getAbsolutePath();
            excelnamefield.setText(excelname);
        }
    } );
    saveButtonFile.setBounds(320, 60, 150, 30);
    
    final JLabel filelabel = new JLabel("Filename (alignment): "); //Create a label to identify which textfield is for the alignmentfilename
        filelabel.setBounds(10, 10, 150, 30);
    final JTextField filenamefield = new JTextField( filename, 40 ); //Create a textfield to write in the alignmentfilename
//    final JTextField filenamefield = new JTextField( "/home/ingo/workspace/musi/bibalign/testdata/test_alignment.fas", 40 ); //Create a textfield to write in the alignmentfilename
    filenamefield.setBounds(170, 10, 150, 30);
    filenamefield.setToolTipText("Insert the name of the alignmentfile"); //Tooltip for the alignmentfile-textfield
    filenamefield.addActionListener( new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
        }
    } ); 
        final JFileChooser filechooser = new JFileChooser();
    filechooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    final JButton openButtonFile = new JButton("Open a fasta file...");
    openButtonFile.addActionListener(new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
            filechooser.showOpenDialog(panel);
            File file = filechooser.getSelectedFile();
            filename = file.getAbsolutePath();
            filenamefield.setText(filename);
        }
    } );
    openButtonFile.setBounds(320, 10, 150, 30);
    
    final JCheckBox table2checkbox = new JCheckBox( "Nucleotide substitutions", true); //A checkbox if table2 should be written
    table2checkbox.setBounds(10, 100, 162, 40);
    table2checkbox.setToolTipText("Create Table 2 - Nucleotide substitutions - all nucleotides that are different from the reference nucleotides");
    final JCheckBox table3checkbox = new JCheckBox( "Codon substitutions", true); //A checkbox if table3 should be written
    table3checkbox.setBounds(10, 130, 155, 40);
    table3checkbox.setToolTipText("Create Table 3 - Codon substitutions - all triplets that are different from the reference triplets");
    final JCheckBox table4checkbox = new JCheckBox( "Amino acid substitutions", true); //A checkbox if table4 should be written
    table4checkbox.setBounds(10, 160, 165, 30);
    table4checkbox.setToolTipText("Create Table 4 - Amino acid substitutions - all amino acids that are different from the reference amino acids");
    final JCheckBox table5checkbox = new JCheckBox( "Amino acid distribution", true); //A checkbox if table5 should be written
    table5checkbox.setBounds(10, 190, 155, 30);
    table5checkbox.setToolTipText("Create Table 5 - Amino acid distribution - calculation of the distribution of all amino acids");
    final JCheckBox table6checkbox = new JCheckBox( "Deletions and changes", true); //A checkbox if table6 should be written
    table6checkbox.setBounds(10, 220, 155, 30);
    table6checkbox.setToolTipText("Create Table 6 - Deletions and changes - different calculations (number deletions, substitutions)");
    final JCheckBox mutsidescheckbox = new JCheckBox( "Use of mutation sites", true); //A checkbox if mutationsides are used
    mutsidescheckbox.setBounds(230, 100, 145, 30);
    mutsidescheckbox.setToolTipText("Do you use mutation sites?");
    mutsidescheckbox.setEnabled(true);
    
    //All advanced settings
    final JLabel advancedlabel = new JLabel("Advanced settings:"); 
    advancedlabel.setBounds(520, 35, 160, 30);
    
    final JPanel advanced_panel = new JPanel(); 
    advanced_panel.setBounds(520, 35, 490, 260);
    advanced_panel.setBorder(BorderFactory.createLineBorder(Color.black));
    advanced_panel.setLayout(null);
    
    final JLabel aminolabel = new JLabel("Codon table: "); //Create a label to identify which textfield is for the aminoacidtable-filename
    aminolabel.setBounds(10, 10, 150, 40);
    final JTextField aminonamefield = new JTextField( aminotablename, 40 ); //Create a textfield to write in the aminoacidtable-filename
//    final JTextField aminonamefield = new JTextField( "/home/ingo/workspace/musi/bibalign/table1.xls", 40 ); //Create a textfield to write in the aminoacidtable-filename
    aminonamefield.setBounds(155, 10, 150, 30);
    aminonamefield.setToolTipText("Insert the name of the amino acids table"); //Tooltip for the aminoacidtable-textfield
    aminonamefield.addActionListener( new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
        }
    } ); 
        final JFileChooser aminochooser = new JFileChooser();
    aminochooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    final JButton openButtonAmino = new JButton("Open an excel file...");
    openButtonAmino.addActionListener(new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
            aminochooser.showOpenDialog(panel);
            File file = aminochooser.getSelectedFile();
            aminotablename = file.getAbsolutePath();
            aminonamefield.setText(aminotablename);
        }
    } );
    openButtonAmino.setBounds(305, 10, 150, 30);

    final JLabel deletionslabel = new JLabel("Display of deletions in the tables:");
    deletionslabel.setBounds(10, 55, 250, 30);
    
    final JLabel singledeletionlabel = new JLabel("Single deletion: "); //Create a label to identify which textfield is for the aminoacidtable-filename
    singledeletionlabel.setBounds(10, 95, 150, 30);
    final JTextField singledeletionfield = new JTextField( singledeletion, 40 ); //Create a textfield to write in the aminoacidtable-filename
    singledeletionfield.setBounds(155, 95, 50, 30);
    singledeletionfield.setToolTipText("Insert the sign for deletions"); //Tooltip for the aminoacidtable-textfield
    singledeletionfield.addActionListener( new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
        }
    } );  
    final JLabel doubledeletionlabel = new JLabel("Double deletion: "); //Create a label to identify which textfield is for the aminoacidtable-filename
    doubledeletionlabel.setBounds(10, 145, 150, 30);
    final JTextField doubledeletionfield = new JTextField( doubledeletion, 40 ); //Create a textfield to write in the aminoacidtable-filename
    doubledeletionfield.setBounds(155, 145, 50, 30);
    doubledeletionfield.setToolTipText("Insert the sign for deletions"); //Tooltip for the aminoacidtable-textfield
    doubledeletionfield.addActionListener( new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
        }
    } ); 
    final JLabel trimerdeletionlabel = new JLabel("Trimer deletion: "); //Create a label to identify which textfield is for the aminoacidtable-filename
    trimerdeletionlabel.setBounds(10, 195, 150, 30);
    final JTextField trimerdeletionfield = new JTextField( trimerdeletion, 40 ); //Create a textfield to write in the aminoacidtable-filename
    trimerdeletionfield.setBounds(155, 195, 50, 30);
    trimerdeletionfield.setToolTipText("Insert the sign for deletions"); //Tooltip for the aminoacidtable-textfield
    trimerdeletionfield.addActionListener( new ActionListener() { 
        @Override public void actionPerformed( ActionEvent e ) { //Does nothing but has to be there
        }
    } );
 
    final JLabel warningslabel = new JLabel("Warnings and Errors:"); 
    warningslabel.setBounds(10, 305, 150, 40);
    warnings.setBounds(10, 345, 800, 370);
    final JButton close = new JButton("Close"); //Create a button to end the programm
    close.setBounds(840, 440, 150, 30);
    close.addActionListener(new ActionListener() 
    {
        public void actionPerformed(ActionEvent event) 
        {
            System.exit(0); //End programm
        }
    }
    );
  
    JButton workbookbutton = new JButton("Run Musi"); //This button will create the whole excelfile with the tables in it
    workbookbutton.setBounds(840, 360, 150, 30);
    workbookbutton.setToolTipText("Create the selected tables");
    workbookbutton.setEnabled(true);
 
    workbookbutton.addActionListener(new ActionListener() 
    {
        public void actionPerformed(ActionEvent event) {

                    warningmessages = "";
                    warnings.setText("Working");
                    excelname = excelnamefield.getText(); //The name the excelfile/workbook shall have
                    int pos_last_dot = excelname.lastIndexOf('.');
                    if (pos_last_dot == -1) 
                    	excelname += ".xls";
                    else {
                    	if (!excelname.substring(pos_last_dot + 1).equals("xls"))
                    		excelname = excelname.substring(0, pos_last_dot) + ".xls";
                    }
                    excelnamefield.setText(excelname);
                    File f = new File(excelname);
                    f.delete();
                    
                    filename = filenamefield.getText(); //The filename of the alignment
                    if(filename.contains(".doc")) {
                        addWarningMessage("Invalid Input - Text-only-format needed. Musi can't handle Wordfiles.");
                    }
                    else {
                        singledeletion = singledeletionfield.getText(); //Get the representation of a single deletion
                        doubledeletion = doubledeletionfield.getText(); //Get the representation of a double deletion
                        trimerdeletion = trimerdeletionfield.getText(); //Get the representation of a trimer deletion

                   
                    
                        String data = filename +System.getProperty("line.separator")+
                                excelname +System.getProperty("line.separator")+
                                aminotablename+System.getProperty("line.separator")+
                                singledeletion+System.getProperty("line.separator")+
                                doubledeletion+System.getProperty("line.separator")+
                                trimerdeletion;
                    
                        Writer fw = null;
                        try
                        {
                            fw = new FileWriter( "musi.config" );
                            fw.write( data );
                            fw.append( System.getProperty("line.separator") ); // e.g. "\n"
                        }
                        catch ( IOException e ) {
                            addWarningMessage( "Konnte Config-Datei nicht erstellen - Eingaben sind beim nächsten Aufruf nicht verfügbar." );
                        }
                        finally {
                            if ( fw != null )
                            try {
                                fw.close();
                            } catch ( IOException e ) {
                            }
                        }
                    
                        Virtual_Sheet virsheet = new Virtual_Sheet();
                        java.lang.Boolean use_mutsides = mutsidescheckbox.isSelected() ? true : false;
                            //If the user uses mutationsides, else the boolean stays on false

                        if (table4checkbox.isSelected() || table5checkbox.isSelected() ||
                            table6checkbox.isSelected()) //If none of the tables is selected which needs aminos it is not nessessary to create them
                        {
                            aminotablename = aminonamefield.getText(); //The name of the used amino acids table
                            Used_Amino_Acids used_aminos = new Used_Amino_Acids(aminotablename); //initialises the list of used amino acids.
                            Amino_Acid.init(used_aminos); //Sets the list from amino acids
                        }

                        Alignment align = new Alignment(filename, use_mutsides); //Ititialise the alignment
                        if(align.isLegal()) {
                            try {
                                Moities moities = new Moities(align, use_mutsides);

                                WritableWorkbook workbook = Workbook.createWorkbook(new java.io.File(excelname));
                                if (table2checkbox.isSelected()) {
                                    Table_Writer.nucleotide_substitutions(align, workbook, singledeletion, use_mutsides, virsheet); //Table 2
                                    addWarningMessage("Nucleotide substitutions done");
                                }
                                if (table3checkbox.isSelected()) {
                                    Table_Writer.codon_substitutions(moities, workbook,
                                    singledeletion, doubledeletion, trimerdeletion, use_mutsides, virsheet); //Table 3
                                    addWarningMessage("Codon substitutions done");

                                }
                                if (table4checkbox.isSelected()) {
                                    Table_Writer.amino_acid_substitutions(moities, workbook, use_mutsides,
                                    singledeletion, doubledeletion, trimerdeletion, virsheet); //Table 4
                                    addWarningMessage("Amino acid substitutions done");
                                }
                                if (table5checkbox.isSelected()) {
                                    Table_Writer.amino_acid_distribution(moities, workbook, use_mutsides,
                                    singledeletion, doubledeletion, trimerdeletion, virsheet); //Table 5
                                    addWarningMessage("Amino acid distribution done");
                                }
                                if (table6checkbox.isSelected()) {
                                    Table_Writer.deletions_and_changes(moities, workbook, use_mutsides,
                                    singledeletion, doubledeletion, trimerdeletion, virsheet); //Table 6
                                    addWarningMessage("Deletions and changes done");
                                }

                                workbook.write();
                                workbook.close();
                                addWarningMessage(excelname + " done\nEnded building!");
                            } catch (IOException ex) {
                                GUI.addWarningMessage("Unknown IOException occured: " + ex);
                            } catch (WriteException ex) {
                                GUI.addWarningMessage("Unknown WriteException occured: " + ex);
                            } catch (IllegalArgumentException ex) {
                                GUI.addWarningMessage("Unknown IllegalArgumentException occured: " + ex);
                            } catch (Exception ex) {
                                GUI.addWarningMessage("Unknown Exception occured: " + ex);
                            }
                        warnings.setText(warningmessages);
                  
                        Amino_Acid.init(null); //Resets the list back to null so for the next table a different aminotable could be used
                    } else {
                        addWarningMessage("Stopped building tables - illegal alignment");
                        Amino_Acid.init(null); //Resets the list back to null so for the next table a different aminotable could be used
                }
            }
        }
    }
    );
   
    /*Adds all buttons, labels and other stuff*/
    panel.add(workbookbutton);
    panel.add(close);

    panel.add(warningslabel);
    panel.add(warnings);
    
    panel.add(input_data_panel);
    panel.add(inputlabel);
    input_data_panel.add(excellabel);
    input_data_panel.add(excelnamefield);
    input_data_panel.add(filelabel);
    input_data_panel.add(filenamefield);
    input_data_panel.add(openButtonFile);
    input_data_panel.add(saveButtonFile);
    input_data_panel.add(table2checkbox);
    input_data_panel.add(table3checkbox);
    input_data_panel.add(table4checkbox);
    input_data_panel.add(table5checkbox);
    input_data_panel.add(table6checkbox);
    input_data_panel.add(mutsidescheckbox);
    
    panel.add(advanced_panel);
    panel.add(advancedlabel);
    advanced_panel.add(aminolabel);
    advanced_panel.add(aminonamefield);
    advanced_panel.add(openButtonAmino);
    advanced_panel.add(singledeletionfield);
    advanced_panel.add(doubledeletionfield);
    advanced_panel.add(trimerdeletionfield);
    advanced_panel.add(singledeletionlabel);
    advanced_panel.add(doubledeletionlabel);
    advanced_panel.add(trimerdeletionlabel);
    advanced_panel.add(deletionslabel);

  }
    
    public static void addWarningMessage(String message) {
        warningmessages += message + "\n";
                        warnings.setText(warningmessages);

    }
    
    public String[] readConfigFile() {
    String file = null; //Will get the file
    StringBuffer input = new StringBuffer();
    String line = null; //Will get a single line of the file
    try {
            BufferedReader buffreader = new BufferedReader(new FileReader("musi.config"));
            while ((line = buffreader.readLine()) != null) //read
            file = (input.append(System.getProperty("line.separator")).append(line)).toString(); //and put it in the input

        } catch (FileNotFoundException ex) {
            GUI.addWarningMessage("File " + filename +" not found"); 
        } catch (IOException ex) {
            GUI.addWarningMessage("Unknown IOException occured: " + ex);   
        } 
    String[] inputNames;
    try {
        inputNames = file.split(System.getProperty("line.separator"));
    } catch (NullPointerException ex) {
            GUI.addWarningMessage("Confic-data couldn't be found. Starting with default inputs.");
            return null;
        }

    return inputNames;
    }
}
