package bibalign;

/**The moities given in the alignment
 *
 * @author Daniela Buchwald
 */
public class Moities {
    private Alignment alignment; //The used alignment
    private Triplet[][] triplets;  //Gets the mutagenized triplets from every sequence
    
    /**Construct moities from alignment
     * 
     * @param alignment The alignment which is used
     */
    public Moities(Alignment alignment, Boolean use_mutsides)
    {
        /**This part puts the alignment in the variable alignment*/
        this.alignment = alignment; //Sets the used alignment
        
        /**This part puts the mutagenized triplets in the variable triplets*/
        int num_entrys = this.get_number_of_all_sequences(); //Get the number of sequences
        int num_mut_pos = this.get_num_mutagenized_positions(use_mutsides);
            //get how much positions was mutagenzied
        
        int mut_pos[] = this.get_mutagenized_positions(num_mut_pos, use_mutsides); //save all positions of mut. pos.

        triplets = new Triplet[num_entrys][num_mut_pos]; //make a new Triplet-matrix
        for(int k = use_mutsides ? 1 : 0; k < num_entrys; k++) //goes throught all sequences. k starts at 1 because 0 is the mut-seq
        {
            for(int j=0; j < num_mut_pos; j++) //for each position which is mutagenzied
            {
                int actual_pos = mut_pos[j];
                triplets[k][j] = this.get_triplet(k, actual_pos); //save the triplet
            }
        }
    }
    
    /**Get triplet at given coordinates
     * 
     * @param no_seq The number of the sequence the triplet should be get from
     * @param pos The position in the sequence where the triplet shall come from
     * @return The requested triplet
     * @throws IllegalArgumentException
     */
    public Triplet get_triplet(int no_seq, int pos)
            throws IllegalArgumentException
    {
        int num_entrys = this.get_number_of_all_sequences(); //get the number of sequences
        int num_nucleos =  alignment.number_of_nucleotides();
        char[] char_triplet = new char[3]; //Will get the characters the triplet will contain
        Triplet triplet = null;
        
        if(no_seq >= num_entrys || pos >= num_nucleos) //Check if the given coordinates are legal, if not:
        {
            throw new IllegalArgumentException("No triplet at the given position. Seq-no:" + no_seq +", tripletpos:" + pos);
        }
        else //If the coordinates are legal
        {
            char_triplet[0] = alignment.get_character(no_seq, pos); //Get the first char of the triplet
            char_triplet[1] = alignment.get_character(no_seq, pos+1); //Get the second char of the triplet
            char_triplet[2] = alignment.get_character(no_seq, pos+2); //Get the thrid char of the triplet

            String char_triplet_string = String.valueOf(char_triplet[0]) + String.valueOf(char_triplet[1])
                + String.valueOf(char_triplet[2]); /**Makes a string from the chars and
                * represents the triplet as a string. Needed for the triplet-constructor */
            triplet = new Triplet(char_triplet_string); //Get the triplet
            //TODO: unlegal triplet -> notice
        }
            return triplet;
    }

    /**Get amino acid at given coordinates
     * 
     * @param no_seq The number of the sequence the amino acid should be get from
     * @param pos The position in the sequence where the amino acid shall come from
     * @return The requested amino acid
     * @throws IllegalArgumentException
     */
    public Amino_Acid get_amino_acid(int no_seq, int pos)
            throws IllegalArgumentException
    {
            Amino_Acid amino = null; //Get the amino acid.
            amino = new Amino_Acid(get_triplet(no_seq, pos));
            return amino;        
    }
    
    /**Get mutagenized positions
     * 
     * @param no Sets how much mutagenized position shall be found
     * @return An array which contains the positions of all mutagenzied positions
     */
    public int[] get_mutagenized_positions(int no, Boolean use_mutsides)
    {
         /*This part gets the number and the location of the mutagenized positions...*/
        int mut_pos[] = new int[no]; //Will get all mutagenized positions
        int q = 0; //Goes through the array of mutagenized positions 
        int num_of_nucleotides = this.alignment.number_of_nucleotides(); //Gets how much nucleotides are in the sequence  
        
        for(int i=0;i < num_of_nucleotides;i++) //Gets all mut. positions
         {
             if(use_mutsides) //If mutagenized sites are used
             {
                 if((num_of_nucleotides-i)>=3) //If a correct number of nucleotides is in the file (-> triplets)
                 {
                     if(alignment.get_character(0 , i) == 'N' && alignment.get_character(0 , i+1) == 'N' 
                         && alignment.get_character(0 , i+2) == 'N') // if there is a "NNN" (symbol for a mut. pos.) 
                     {
                         mut_pos[q] = i; //Get where the "NNN" is
                         q++; //Go to next array-field to save the next position
                         i+=2; //Is needed so two (or more) following "NNN" will only be found as two
                     }
                 }
             } else {
                 if((num_of_nucleotides-i)>=3) //If a correct number of nucleotides is in the file (-> triplets)
                 {
                     mut_pos[q] = i; //Get where the triplet is
                     q++; //Go to next array-field to save the next position
                     i+=2; //Is needed so two (or more) following "NNN" will only be found as two
                 }
             }
         }
        return mut_pos;
    }
    
    /**Get number of mutagenized positions
     * 
     * @return How many mutagenzied positions are in the sequences
     */
    public int get_num_mutagenized_positions(java.lang.Boolean use_mutsides)
    { 
        if(use_mutsides) { //If mutagenized positions are used
        int no = 0; //Counts the number of mut. positions
        int num_of_nucleotides = this.alignment.number_of_nucleotides(); //Get how many nucleotides are in the sequences
            
        if(num_of_nucleotides%3 == 0) //If there is at least a triplet
        {
            for(int i=0; i < num_of_nucleotides; i++) //Count all mut. positions
            {
                if(alignment.get_character(0 , i) == 'N' && alignment.get_character(0 , i+1) == 'N'
                 && alignment.get_character(0 , i+2) == 'N') //if there is a "NNN" (symbol for mutagenzied position
                {
                    no++; //Number of found mut. positions +1
                    i += 2; // Is needed so two (or more) following "NNN" will only be found as two
                }
            }
        }
        else
            GUI.addWarningMessage("Warning: Last nucleotides are no triplet");
            
        return no;
        }
        else { //If mutagenized positions are not used
            int num_nucleos = this.alignment.number_of_nucleotides(); //Set the number of nucleotides
            return num_nucleos/3; //Calculates how many triplets are in the file
        }            
    }
    
    /**Gets the sequencenames
     * 
     * @param no The number of the sequence the name is requested from
     * @return The requested name
     * @throws IllegalArgumentException
     */
    public String get_seq_names(int no)
            throws IllegalArgumentException
    {
        int num_entrys = this.get_number_of_all_sequences(); //get the number of sequences
        if(no >= num_entrys) //Check if the coordinates are legal
        {
            throw new IllegalArgumentException("Unlegal number of sequence");
        }
        return alignment.get_seq_name(no); //Return the requested names
    }
    
     /**Returns the number of entrys
     * 
     * @return How many entrys/sequences are in the alignment
     */
    public int get_number_of_all_sequences() 
    {
        return alignment.number_of_all_sequences(); //Return how many entrys/sequences are in the alignment
    }
    
    /**Returns the mutagenized position
     * which is requested. 
     * 
     * @param seq_no The number of the sequence
     * @param pos The position of the mut.pos.
     * @return The requested position
     * @throws IllegalArgumentException
     */
    public Triplet get_mut_position(int no_seq, int pos)
            throws IllegalArgumentException
    {
        int num_entrys = this.get_number_of_all_sequences(); //Get the number of sequences
        int num_nucleos =  alignment.number_of_nucleotides(); //Get the number of nucleotides in the file
        
        if(no_seq >= num_entrys || pos >= num_nucleos) //Check whether the coordinates are ok
        {
            throw new IllegalArgumentException("No triplet at the given position. Seq-no:" + no_seq +", tripletpos:" + pos);
        }        
        return triplets[no_seq][pos]; //Return the requested position
    }
    
    /**Gets the alignment
     * 
     * @return The used alignment
     */
    public Alignment get_alignment()
    {
        return alignment; //Return the used alignment
    }
}
