
package bibalign;

import jxl.write.*; 
import com.mallardsoft.tuple.*;
import java.util.*;

/**Static class containing functions to generate Table 2-5
 *
 * @author Daniela Buchwald
 */
public class Table_Writer {
    
    private static int MAX_ROW = 65536; //TODO: COrrect value?

    private static int MAX_COL = 256;

    /**Makes a format for the text (arial, size 8) */
        private static final WritableFont FONT_FOR_TABLE = new WritableFont(WritableFont.ARIAL, 10);
        private static final WritableCellFormat FORMAT_FOR_TABLE = new WritableCellFormat (FONT_FOR_TABLE);
        private static java.lang.Boolean withoutExceptions = true;
        
    /**Create Table 2: Nucleotide substitutions
     * 
     * @param alignment The used Alignment
     * @param workbook The file where the table is written in
     */
    public static void nucleotide_substitutions(Alignment alignment, WritableWorkbook workbook, String deletion, java.lang.Boolean use_mutsides, Virtual_Sheet virsheet)
    {        
        final int SHEETNUMBER = 0;
        /*Makes a new sheet named "Table 2"*/
        WritableSheet sheet = workbook.createSheet("Nucleotide substitutions", SHEETNUMBER);
        
        /*Constants for the strings in the table**/
        final String STR_TITLE = "Nucleotide substitutions";
        final String STR_NUM_NUCLEO = "Number of nucleotide";
        final String STR_TYPE_NUCLEO = "Type of ref. nucleotide";
        final String STR_SEQ_NAME = "Name of sequence";
        final String STR_SUBST = "Substitutions ";
        
        /*Constants for the table - which column and row is used**/
        final int COL_TITLE = 1;
        final int ROW_TITLE = 0;
        final int COL_NUM_NUCL = 0;
        final int COL_NUCLEOS_AND_NAME = COL_NUM_NUCL + 1;
        final int ROW_NUM_NUCLEO = 3;
        final int ROW_TYPE_NUCLEO = 3;
        final int ROW_SEQ_NAME = 2;
        final int COL_SUBST = 2;
        final int ROW_SUBST = 3;
        
        /*Initalises the labels of the table**/
        virsheet.add_cell(COL_TITLE, ROW_TITLE, STR_TITLE, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_NUM_NUCL, ROW_NUM_NUCLEO, STR_NUM_NUCLEO, FORMAT_FOR_TABLE, false, 17);
        virsheet.add_cell(COL_NUCLEOS_AND_NAME, ROW_TYPE_NUCLEO, STR_TYPE_NUCLEO, FORMAT_FOR_TABLE, false, 17);
        virsheet.add_cell(COL_NUCLEOS_AND_NAME, ROW_SEQ_NAME, STR_SEQ_NAME, FORMAT_FOR_TABLE, false, 17);
        virsheet.add_cell(COL_SUBST, ROW_SUBST, STR_SUBST, FORMAT_FOR_TABLE, false);


        int num_of_nucleotides = alignment.number_of_nucleotides();
        int ind_wildtype = use_mutsides ? 1 : 0;

        for(int i=0; i<num_of_nucleotides;i++) //Go through all nucleotides
        {
            int str_num_nucleo = i+1; //The current nucleotide-number
            int row_num_nucleo = i+1; //The column the nucleotide-number should be written
            char wildtype_char = alignment.get_character(ind_wildtype, i); //The base of the wildtype at the current position
            String str_wildtype = String.valueOf(wildtype_char); //Make a string from the char                 
            virsheet.add_cell(COL_NUM_NUCL, row_num_nucleo+3, String.valueOf(str_num_nucleo), FORMAT_FOR_TABLE, true); //TODO: Add parameter to seperate Label and Number??
            virsheet.add_cell(COL_NUCLEOS_AND_NAME, row_num_nucleo+3, str_wildtype, FORMAT_FOR_TABLE, false);            
        }
        
        int num_all_seq = alignment.number_of_all_sequences(); //Get how much sequences are used (incl. mutagenzied and wildtype)
        
        for(int l=ind_wildtype + 1; l< num_all_seq; l++) //Go through all sequences without the first two (mut. and wildtype) or one (if none mut. is there)
        {
            String seq_name = alignment.get_seq_name(l); //Get the current seq. name
            int col_seq_name = use_mutsides ? l : l+1; //The row where the name should be written in    
            virsheet.add_cell(col_seq_name, ROW_SEQ_NAME, seq_name, FORMAT_FOR_TABLE, false);
        }
                
        for(int j=ind_wildtype + 1; j<num_all_seq;j++) //Go through all sequences but the first two (mut and wildtype) or one (if none mut. is there)
        {
            for(int h=0; h<num_of_nucleotides;h++) //Go through all nucleotides
            {
                char wildtype_char = alignment.get_character(ind_wildtype, h); //Get the wildtype-base to compare with
                char current_char = alignment.get_character(j, h); //Get the current-case to compare to the wildtype
                String char_that_differs = null; //Reset the String so it is empty
                int row_char_that_differs = h+4; 
                int col_char_that_differs = use_mutsides ? j : j+1;
                if(current_char == '-') //If the current char is a gap a comparison is not necessary
                {
                    char_that_differs = deletion; //Put the symbol for a deletion in
                    virsheet.add_cell(col_char_that_differs,row_char_that_differs, char_that_differs, FORMAT_FOR_TABLE, false );
                }
                else if(wildtype_char != current_char) //If the bases are different
                {
                    char_that_differs = String.valueOf(current_char); //Write the base that differs
                    virsheet.add_cell(col_char_that_differs,row_char_that_differs, char_that_differs, FORMAT_FOR_TABLE, false );
                }
            }
        }        
        try {
            virsheet.write_to_excel_sheet(sheet, MAX_ROW, MAX_COL, false);
        } catch (WriteException ex) {
            GUI.addWarningMessage("Error while writing the virtual_sheet - nucleotide substitutions is not included");
        }
    }
    
    /**Create Table 3: Codon Substitutions
     * 
     * @param moities The used moitiesfile
     * @param workbook The file where the table is written in
     */
    public static void codon_substitutions(Moities moities, WritableWorkbook workbook, 
            String single_del, String double_del, String tri_del, java.lang.Boolean use_mutsides, Virtual_Sheet virsheet)
    {       
        /*Makes a new sheet named "Table 3"*/
        final int SHEETNUMBER = 1;
        WritableSheet sheet = workbook.createSheet("Codon substitutions", SHEETNUMBER);
        
        /*Constants for the strings in the table**/
        final String STR_CODON_SUBST = "Codon substitutions";
        final String STR_MUT_POS = "Mutagenazied positon";
        final String STR_WILD_CODON = "Wildtype codon";
        
        /*Constants for the table - which column and row is used**/
        final int COL_TITLE = 1;
        final int ROW_TITLE = 0;
        final int COL_MUT_POS = 0; 
        final int ROW_MUT_POS = 1;
        final int COL_WILD_CODON = 0;
        final int ROW_WILD_CODON = 2;
        
        /*Initalises the labels of the table**/
        virsheet.add_cell(COL_TITLE, ROW_TITLE, STR_CODON_SUBST, FORMAT_FOR_TABLE, false, 15);
        virsheet.add_cell(COL_MUT_POS, ROW_MUT_POS, STR_MUT_POS, FORMAT_FOR_TABLE, false, 16);
        virsheet.add_cell(COL_WILD_CODON, ROW_WILD_CODON, STR_WILD_CODON, FORMAT_FOR_TABLE, false, 12); 

        /*Adds the number of mutagenzied positions in a row and the wildtype-codons*/
        int num_mut_pos = moities.get_num_mutagenized_positions(use_mutsides);
        int ind_wildtype = use_mutsides ? 1 : 0;
                
        for(int i=0; i<num_mut_pos; i++) //Go through all mutagenzied positions
        {
            Triplet triplet = moities.get_mut_position(ind_wildtype, i); //Get the triplet of the wildtype
            String current_triplet = triplet.toString(); //Make a string out of the triplet
            int col_mut_and_wild_codons = i+1; //The column where to write these two things
            virsheet.add_cell(col_mut_and_wild_codons, ROW_MUT_POS, String.valueOf(i+1), FORMAT_FOR_TABLE, true);
            virsheet.add_cell(col_mut_and_wild_codons, ROW_WILD_CODON, current_triplet, FORMAT_FOR_TABLE , false);
        }
            
        /*Adds the sequence names to the table*/
        int number_sequences = moities.get_number_of_all_sequences();
        final int COL_SEQ_NAMES = 0; //Which column should be used for the sequence names
        for(int j=ind_wildtype + 1; j< number_sequences;j++) //Go through all sequences but the first two (mut and wildtype) or one (if none mut. is used)
        {
            int row_seq_names = use_mutsides ? j+1 : j+2;
            String seq_name = moities.get_seq_names(j); //Get the current sequence name
            virsheet.add_cell(COL_SEQ_NAMES, row_seq_names, seq_name, FORMAT_FOR_TABLE, false);
                //First column but start at the fourth line (l+4)
            for(int h=0;h<num_mut_pos;h++) //Go through all mutagenzied positions
            {
                Triplet triplet = moities.get_mut_position(j, h); //Get the acutal triplet
                Triplet wildtype_tripet = moities.get_mut_position(ind_wildtype, h);//Get the wildtype triplet
                
                if(!triplet.toString().equals(wildtype_tripet.toString())) //If the triplets are not the same
                {
                    String triplet_str = triplet.toString(); //Make a string from the triplet
                    int col_triplet_differs = h+1;
                    int row_triplet_differs = use_mutsides ? j+1 : j+2;;
                    char[] triplet_chars = triplet_str.toCharArray(); //Make a char-array from the triplet(-string)
                        
                    if(java.util.regex.Pattern.compile("---").matcher(triplet_str).find()) //If there is a trimer deletion
                    {
                        triplet_str = tri_del;
                    }
                    else if(java.util.regex.Pattern.compile("--").matcher(triplet_str).find()) //If there is a single deletion
                    {
                        int index_doub_del = triplet_str.indexOf("--");
                        if(index_doub_del == 0)
                            triplet_str = double_del + String.valueOf(triplet_chars[2]);
                        if(index_doub_del == 1)
                            triplet_str = String.valueOf(triplet_chars[0]) + double_del;
                    }
                    else if(java.util.regex.Pattern.compile("-").matcher(triplet_str).find()) //If there is at least one single deletion
                    {
                        String triplet_chars_0 = String.valueOf(triplet_chars[0]); //Get the first base
                        String triplet_chars_1 = String.valueOf(triplet_chars[1]); //Get the second base
                        String triplet_chars_2 = String.valueOf(triplet_chars[2]); //Get the third base
                            
                        if(triplet_chars[0] == '-') //If the first char is a gap
                            triplet_chars_0 = single_del; //Save the string which represents a single del.
                        if(triplet_chars[1] == '-') //If the second char is a gap
                            triplet_chars_1 = single_del; //Save the string which represents a single del.
                        if(triplet_chars[2] == '-') //If the third char is a gap
                            triplet_chars_2 = single_del; //Save the string which represents a single del.
                        triplet_str = triplet_chars_0 + triplet_chars_1 + triplet_chars_2; //Merge all strings.
                    }
                    virsheet.add_cell(col_triplet_differs, row_triplet_differs, triplet_str, FORMAT_FOR_TABLE, false);
                }
            }
        }
        try {
            virsheet.write_to_excel_sheet(sheet, MAX_ROW, MAX_COL, false);
        } catch (WriteException ex) {
            GUI.addWarningMessage("Error while writing the virtual_sheet - codon substitutions is not included");
        }
    }
    
    /**Create Table 4: Amino acid substitutions
     * 
     * @param moities The used moitiesfile
     * @param workbook The file where the table is written in
     */
    public static void amino_acid_substitutions(Moities moities, WritableWorkbook workbook, java.lang.Boolean use_mutsides,
            String single_deletion, String double_deletion, String trimer_deletion, Virtual_Sheet virsheet)
    {
        /*Makes a new sheet named "Table 4"*/
        final int SHEETNUMBER = 2;
        WritableSheet sheet = workbook.createSheet("Amino acid substitutions", SHEETNUMBER); 
        int num_of_mut_pos = moities.get_num_mutagenized_positions(use_mutsides); //Gets the Number of mutagenized positions
        
        /*Adds all labels which are known from the begin on*/
        final String STR_TITLE = "Amino acid substitutions per sequence";
        final String STR_SEQ_NAME = "Name of sequence";
        final String STR_GENE_POS = "Gene position";
        final String STR_NUM_SUBST = "No. of substitutions";
        final String STR_NUM_DELS_MUT = "Deletions in mutagenized positions";
        final String STR_NUM_TRI_DELS_MUT = "No. of trimer deletions at mut. positions";
        final String STR_NUM_SINGLE_DELS_MUT = "No. of single deletions at mut. positions";
        final String STR_NUM_DOUB_DELS_MUT= "No. of double deletions at mut. positions";
        final String STR_DELS_ELSE = "Deletions elsewhere";
        final String STR_NUM_TRI_DELS_ELSE = "No. of trimer deletions elsewhere";
        final String STR_NUM_SINGLE_DELS_ELSE = "No. of single deletions elsewhere";
        final String STR_NUM_DOUB_DELS_ELSE = "No. of double deletions elsewhere";
        final String STR_NUM_X = "No. of X (unknown amino acids)";
        
        final int ROW_HEADER = 1;
        final int COL_FIRST_SUBST = 1;
        final int COL_FIRST_AFTER_SUBST = COL_FIRST_SUBST + num_of_mut_pos;
        final int COL_TITLE = 1;
        final int ROW_TITLE = 0;
        final int COL_SEQ_NAMES = 0;
        final int ROW_SEQ_NAMES_TITLE = 1;
        final int COL_GENE_POS = 0;
        final int ROW_GENE_POS = 3;
        final int COL_NUM_SUBST = COL_FIRST_AFTER_SUBST;
        final int COL_NUM_DELS_MUT = COL_FIRST_AFTER_SUBST + 1;
        final int COL_NUM_TRI_DELS_MUT = COL_FIRST_AFTER_SUBST + 1;
        final int COL_NUM_SINGLE_DELS_MUT = COL_NUM_TRI_DELS_MUT + 1;
        final int COL_NUM_DOUB_DELS_MUT = COL_NUM_SINGLE_DELS_MUT + 1;
        final int COL_NUM_DELS_ELSE = COL_NUM_DOUB_DELS_MUT + 1;
        final int COL_NUM_TRI_DELS_ELSE = COL_NUM_DOUB_DELS_MUT + 1;
        final int COL_NUM_SINGLE_DELS_ELSE = COL_NUM_TRI_DELS_ELSE + 1;
        final int COL_NUM_DOUB_DELS_ELSE = COL_NUM_SINGLE_DELS_ELSE + 1;
        final int COL_NUM_X = COL_NUM_DOUB_DELS_ELSE + 1;
        

        virsheet.add_cell(COL_NUM_TRI_DELS_MUT, ROW_HEADER, STR_NUM_TRI_DELS_MUT, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_NUM_SINGLE_DELS_MUT, ROW_HEADER, STR_NUM_SINGLE_DELS_MUT, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_NUM_DOUB_DELS_MUT, ROW_HEADER, STR_NUM_DOUB_DELS_MUT, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_NUM_TRI_DELS_ELSE, ROW_HEADER, STR_NUM_TRI_DELS_ELSE, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_NUM_SINGLE_DELS_ELSE, ROW_HEADER, STR_NUM_SINGLE_DELS_ELSE, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_NUM_DOUB_DELS_ELSE, ROW_HEADER, STR_NUM_DOUB_DELS_ELSE, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_NUM_DELS_MUT, ROW_HEADER - 1, STR_NUM_DELS_MUT, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_SEQ_NAMES, ROW_SEQ_NAMES_TITLE, STR_SEQ_NAME, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_NUM_SUBST, ROW_HEADER, STR_NUM_SUBST, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_TITLE, ROW_TITLE, STR_TITLE, FORMAT_FOR_TABLE, false, 25);
        virsheet.add_cell(COL_NUM_X, ROW_HEADER, STR_NUM_X, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_GENE_POS, ROW_GENE_POS, STR_GENE_POS, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_NUM_DELS_ELSE, ROW_HEADER - 1, STR_DELS_ELSE, FORMAT_FOR_TABLE, false);
        

         /*Writes the number of the mutagenized position in the second line*/
         final int ROW_NUMBER_MUT_POS = 1;
        
         for(int i = COL_FIRST_SUBST; i < COL_FIRST_SUBST + num_of_mut_pos; i++) /*This loop wWrites the number of mut. positions in a line
              * *Starts at one because zero is not a valid position to be 
              * examined and the label starts in line one (second line)*/
         {
             int col_number_mut_pos = i;
             virsheet.add_cell(col_number_mut_pos, ROW_NUMBER_MUT_POS, String.valueOf(i), FORMAT_FOR_TABLE, true);
         }
            
         /*Writes the positions of the mutagenizied positions in the third line*/
         int mut_positions[] = new int[num_of_mut_pos]; //Will get all positions which are mutagenized
         mut_positions = moities.get_mutagenized_positions(num_of_mut_pos, use_mutsides); //get the mut.pos
         final int ROW_POS_OF_MUT_POS = 3;
        
         for(int j=1; j <= num_of_mut_pos;j++)/*Writes the positions in a line.
            * Starts at one because there should be no entry at position zero*/
         {
             int current_mut_pos = mut_positions[j-1]+1;
             int col_pos_of_mut_pos = j;
             virsheet.add_cell(col_pos_of_mut_pos, ROW_POS_OF_MUT_POS, String.valueOf(current_mut_pos), FORMAT_FOR_TABLE, true);
         }
        
         /*Writes the sequence names in a column*/
         int num_of_all_sequences = moities.get_number_of_all_sequences(); /*Gets the number of entrys 
             * so the while-loop knows how often it has to repeat*/
         int ind_wildtype = use_mutsides ? 1 : 0;
                    
         for(int l=ind_wildtype + 1; l < num_of_all_sequences; l++)/*Writes the sequencenames in a column.
             * Starts at one if there is only a wildtypeseq and no mut-sides 
             * and at two if there is both.*/
         { 
             int row_seq_names = l+2;
             String current_seq_name = moities.get_seq_names(l);
             virsheet.add_cell(COL_SEQ_NAMES, row_seq_names, current_seq_name, FORMAT_FOR_TABLE, false);
                 //First column but start at the fourth line (l+4)
         }
                   
         /*Writes the amino acids of the wild-type in a line*/
         int mut_pos_length = mut_positions.length; //Saves how much mutagenized positions are in the array
         final int ROW_WILD_AMINO = 2;
         final int COLUMN_WIDTH = 5;
        
         for(int k=0; k < mut_pos_length; k++)//This loop puts the aminoacids from the wildtype in a line
         {
             Triplet triplet = moities.get_mut_position(ind_wildtype, k);
             Amino_Acid amino_acid = new Amino_Acid(triplet); //Get the aminoacid which was represented by the triplet
             char amino_char = amino_acid.to_char(); //Get the char which represents the aminoacid
             int col_wild_amino = k+1;
             virsheet.add_cell(col_wild_amino, ROW_WILD_AMINO, String.valueOf(amino_char), FORMAT_FOR_TABLE, false); 
         }
                        
         /*Compairs the amino acids of the other sequences to the 
            *one of the wildtype and writes them in a line if  the differ.
            *Also writes the number of substitutions, trimer/single/double deletions in a column.*/
         String amino_output = ""; /*Will get d1 if the is a single del., d2 if there is a double del., 
            * d3 if there is a trimer del. and d1d1 if there are two single dels.
            * Or will get the amino_char if there was no deletion*/
         int trimer_deletions=0, double_deletions=0, single_deletions=0, num_subs=0;
         int[] num_found_X = new int[num_of_all_sequences-ind_wildtype-1];
         
         for(int m = ind_wildtype + 1; m < num_of_all_sequences; m++) //Goes through all entrys without the wildtype and the information sequence (with the NNN), if there
         {
             for(int n=0; n < mut_pos_length; n++) //Repeat until all mutagenzied positions are checked
             {
                 Triplet wildtype_triplet = moities.get_mut_position(ind_wildtype, n); //Get the triplet from the nth mut.pos from the wildtype-seq
                 Amino_Acid wildtype_amino_acid = new Amino_Acid(wildtype_triplet); //Make a wildtype amino acid out of the wildtype-triplet
                 char wildtype_amino_char = wildtype_amino_acid.to_char(); //Get the char of the wildtyp-as to compare it
                 Triplet triplet = moities.get_mut_position(m, n); //Gets the current triplet from the other sequence
                 Amino_Acid amino_acid = new Amino_Acid(triplet);
                 char amino_char = amino_acid.to_char();
                 Quintuple<String, Integer, Integer,Integer, Integer> char_and_deletions = compare_char(triplet, amino_char, single_deletion,
                     double_deletion, trimer_deletion);                                                 //Checks if the amino char is a deletion (and how much are there), a X or returns the amino if it differs

                if(wildtype_amino_char != amino_char) //Compare wildtype-as with the other as and do something if the differ
                {
                     amino_output = Tuple.get1(char_and_deletions); //Gets the amino char, deletion-symbol or X
                     single_deletions += Tuple.get2(char_and_deletions); //Gets how many single dels are there (1 or 0)
                     double_deletions += Tuple.get3(char_and_deletions); //Gets how many double dels are there (1 or 0)
                     trimer_deletions += Tuple.get4(char_and_deletions); //Gets how many trimer dels are there (1 or 0)
                     num_subs += Tuple.get5(char_and_deletions); //Gets how many substitutions are there (1 or 0)
                }
                if(!amino_acid.is_legal() && Tuple.get2(char_and_deletions) == 0 && Tuple.get3(char_and_deletions) == 0
                  && Tuple.get4(char_and_deletions) == 0)
                    num_found_X[m-ind_wildtype-1]++;
                int col_diff_amino = n+1;
                int row_diff_amino = m+2;
                virsheet.add_cell(col_diff_amino, row_diff_amino, amino_output, FORMAT_FOR_TABLE, false);         
                amino_output = ""; //Reset the output
            }
            int[][] num_dels_elsewhere = number_of_deletions(moities.get_alignment(), "no_file", use_mutsides);
             /*Will get the number of the different deletions on other positions:
             * [x][0]: number of single deletions
             * [x][1]: number of double deletions
             * [x][2]: number of trimer deletions*/
            int row_number_of_everything = m+2;
                                
            /*Makes all labels*/
            virsheet.add_cell(COL_NUM_SUBST, row_number_of_everything, String.valueOf(num_subs), FORMAT_FOR_TABLE, true);
            virsheet.add_cell(COL_NUM_TRI_DELS_MUT, row_number_of_everything, String.valueOf(trimer_deletions), FORMAT_FOR_TABLE, true);
            virsheet.add_cell(COL_NUM_SINGLE_DELS_MUT, row_number_of_everything, String.valueOf(single_deletions), FORMAT_FOR_TABLE, true);
            virsheet.add_cell(COL_NUM_DOUB_DELS_MUT, row_number_of_everything, String.valueOf(double_deletions), FORMAT_FOR_TABLE,true);
            virsheet.add_cell(COL_NUM_SINGLE_DELS_ELSE, row_number_of_everything, String.valueOf(num_dels_elsewhere[m][0]), FORMAT_FOR_TABLE,true);
            virsheet.add_cell(COL_NUM_DOUB_DELS_ELSE, row_number_of_everything, String.valueOf(num_dels_elsewhere[m][1]), FORMAT_FOR_TABLE, true);
            virsheet.add_cell(COL_NUM_TRI_DELS_ELSE, row_number_of_everything, String.valueOf(num_dels_elsewhere[m][2]), FORMAT_FOR_TABLE, true);
            virsheet.add_cell(COL_NUM_X, row_number_of_everything, String.valueOf(num_found_X[m-1-ind_wildtype]), FORMAT_FOR_TABLE,true);
                                
            /*Resets all variables*/
            num_subs = 0;
            trimer_deletions = 0;
            single_deletions = 0;
            double_deletions = 0;
        }
        try {
            virsheet.write_to_excel_sheet(sheet, MAX_ROW, MAX_COL, false);
        } catch (WriteException ex) {
            GUI.addWarningMessage("Error while writing the virtual_sheet - amino acid substitutions is not included");
        }
    }
    
    /**Create Table 5: Amino acids distribution
     * 
     * @param moities The used moitiesfile
     * @param workbook The file where the table is written in
     */
    public static void amino_acid_distribution(Moities moities, WritableWorkbook workbook, java.lang.Boolean use_mutsides, 
                String single_deletion, String double_deletion, String trimer_deletion, Virtual_Sheet virsheet)
    {
        ArrayList<Character> aminolist = Used_Amino_Acids.amino_list();  
        int number_of_aminos = aminolist.size();
        
        /*Makes a new sheet named "Table 5"*/
        final int SHEETNUMBER = 3;
        WritableSheet sheet = workbook.createSheet("Amino acid distribution", SHEETNUMBER);   
        int num_of_mut_pos = moities.get_num_mutagenized_positions(use_mutsides); //Gets the Number of mutagenized positions      
        int num_of_sequences = moities.get_number_of_all_sequences();
        
        final String STR_TITLE = "Amino acid substitutions in all sequences";
        final String STR_AMINO = "Amino acid";
        final String STR_TOTAL = "Total:";
        final String STR_NUM_SUBS = "No. of substitutions";
        final String STR_TRI_DELS_MUT = "No. of trimer deletions in mut.codons";
        final String STR_SINGLE_DELS_MUT = "No. of single deletions in mut.codons";
        final String STR_DOUB_DELS_MUT = "No. of double deletions in mut.codons";
        final String STR_NUM_SEQ =  "No. of sequences";
        
        final int COL_TITLE = 1;
        final int ROW_TITLE = 0;
        final int COL_AMINO = 0;
        final int ROW_AMINO = 2;
        final int COL_TOTAL = num_of_mut_pos + 1;
        final int COL_AMINOEND = COL_TOTAL + 1;
        final int ROW_TOTAL = 1;
        final int COL_SUBST = 0;
        final int COL_DELS = 0;
        final int COL_NUM_SEQ = 0;
        final int ROW_SUBS = number_of_aminos + 5;
        final int ROW_TRI_DELS = ROW_SUBS + 1;
        final int ROW_SINGLE_DELS = ROW_TRI_DELS + 1;
        final int ROW_DOUB_DELS = ROW_SINGLE_DELS + 1;
        final int ROW_NUM_SEQ = ROW_DOUB_DELS + 2;
        
        /*Adds all labels which are known from the begin on*/
        virsheet.add_cell(COL_TITLE, ROW_TITLE, STR_TITLE, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_AMINO, ROW_AMINO, STR_AMINO, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_TOTAL, ROW_TOTAL, STR_TOTAL, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SUBST, ROW_SUBS, STR_NUM_SUBS, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_DELS, ROW_TRI_DELS, STR_TRI_DELS_MUT, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_DELS,ROW_SINGLE_DELS, STR_SINGLE_DELS_MUT, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_DELS, ROW_DOUB_DELS, STR_DOUB_DELS_MUT, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_NUM_SEQ, ROW_NUM_SEQ, STR_NUM_SEQ, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_AMINOEND, ROW_AMINO, STR_AMINO, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(1, ROW_NUM_SEQ, String.valueOf(num_of_sequences-2), FORMAT_FOR_TABLE, true);
         //Has to be num_of_sequences-2 because the first two are the wildtype and the mutagenized positions

        /*Writes the number of the mutagenized position in the second line*/
        final int ROW_MUT_POS = 1;
        
        for(int i=1; i <= num_of_mut_pos; i++) /*Writes the number of mut. positions in a line. 
         * Starts at one because zero is not a valid position to be 
         * examined and the label starts in line one (second line)*/
        {
            int col_mut_pos = i;
            virsheet.add_cell(col_mut_pos, ROW_MUT_POS, String.valueOf(i), FORMAT_FOR_TABLE, true);
            //sheet.setColumnView(col_mut_pos, COLUMN_WIDTH_MUT_POS); //Changed the width of the columns
        }
        int l = 0;
        /*Writes all used amino acids in a column*/
        for(l=0; l < number_of_aminos; l++) //Writes the amino acids in a row
        {
            int row_amino = l+4;
            String str_amino_character = String.valueOf(aminolist.get(l));
            virsheet.add_cell(COL_AMINO, row_amino, str_amino_character, FORMAT_FOR_TABLE, false);
            virsheet.add_cell(COL_AMINOEND, row_amino, str_amino_character, FORMAT_FOR_TABLE, false);
        }
        int row_amino = l+4;
        virsheet.add_cell(COL_AMINO, row_amino, "X", FORMAT_FOR_TABLE, false);
        
        /*Writes the amino acids of the wild-type in a line*/
        int ind_wildtype = use_mutsides ? 1 : 0;
        
        //TODO: Auslagern??
        for(int k=0; k < num_of_mut_pos; k++) //Puts the aminoacids from the wildtype in a line
        {
            Triplet triplet = moities.get_mut_position(ind_wildtype, k);
            Amino_Acid amino_acid = new Amino_Acid(triplet); //Get the aminoacid which was represented by the triplet
            char amino_char = amino_acid.to_char(); //Get the char which represents the aminoacid
            int col_amino = k+1;
            String str_amino_character = String.valueOf(amino_char);
            virsheet.add_cell(col_amino, ROW_AMINO, str_amino_character, FORMAT_FOR_TABLE, false);
            //sheet.setColumnView(col_amino, COLUMN_WIDTH_AMINO); //Some formatation which sets the width of some columns
        }
        /*Fills out the table*/
        String amino_output = ""; //Will get the amino which differs, X or a deletion-symbol
        int single_deletions = 0, double_deletions = 0, trimer_deletions = 0; //Counts the deletions
        int total_single_deletions = 0, total_double_deletions = 0, total_trimer_deletions=0; //Counts the deletions in all sequences on the same mut. pos.
        int[] all_matches_from_a_amino = new int[number_of_aminos]; //Counts the total amino matches in a row
        int matches_for_all_X = 0;
        int all_substitutions = 0; //Counts the total substitutions in a row
        int[] subs_at_mut_pos = new int[num_of_mut_pos]; //Counts all substitutions on a mut.pos.
       
        for(int i=0; i<num_of_mut_pos; i++) //Go through all mutagenzied positions
        {
            for(int j=0; j<number_of_aminos;j++) //Check for every amino
            {
                single_deletions = 0; //Reset the number of deletions (in front so total_single_dels can add them before the reset
                double_deletions = 0; //Reset the number of deletions
                trimer_deletions = 0; //Reset the number of deletions
                int amino_match = 0; //Reset the amino-counter
                int X_match = 0;
                for(int k=ind_wildtype + 1; k<num_of_sequences;k++)
                {
                    Triplet wild_triplet = moities.get_mut_position(ind_wildtype, i); //Get the triplet of the wildtype
                    Amino_Acid wild_amino = new Amino_Acid(wild_triplet); //Make a amino acid from the wildtype-triplet
                    char wild_amino_char = wild_amino.to_char(); //Get the char of the wildtype-amino acid
                    Triplet triplet = moities.get_mut_position(k, i); //Get the triplet to compare with
                    Amino_Acid amino = new Amino_Acid(triplet); //Make a amino from the triplet
                    char amino_char = amino.to_char(); //Get the char of the amino
                    Quintuple<String, Integer, Integer,Integer, Integer> char_and_deletions = null;
                  
                    if(wild_amino_char != amino_char) //If the two chars are not the same
                    {
                        char_and_deletions = compare_char(triplet, amino_char, single_deletion,
                          double_deletion, trimer_deletion);
                        //Checks if the amino char is a deletion (and how much are there), a X or returns the amino if it differs
                        amino_output = Tuple.get1(char_and_deletions); //Gets the amino char, deletion-symbol or X
                        single_deletions += Tuple.get2(char_and_deletions); //Gets how many single dels are there (1 or 0)
                        double_deletions += Tuple.get3(char_and_deletions); //Gets how many double dels are there (1 or 0)
                        trimer_deletions += Tuple.get4(char_and_deletions); //Gets how many trimer dels are there (1 or 0)
                        if(amino_output.equals(String.valueOf(aminolist.get(j)))) //If the current amino-char is the same as the current one from the list
                        {
                            ++amino_match; //Notice that the aminos are the same
                            ++all_matches_from_a_amino[j]; //Save for this amino a match for the total row
                            ++all_substitutions; //Notice that there is a substitution for the total row
                            ++subs_at_mut_pos[i]; //Notice that there is a substitution for this mutagenized position
                        }
                    } 
                    if(!amino.is_legal())
                    {
                        char_and_deletions = compare_char(triplet, amino_char, single_deletion,
                            double_deletion, trimer_deletion);
                        single_deletions = Tuple.get2(char_and_deletions); //Gets how many single dels are there (1 or 0)
                        double_deletions = Tuple.get3(char_and_deletions); //Gets how many double dels are there (1 or 0)
                        trimer_deletions = Tuple.get4(char_and_deletions); //Gets how many trimer dels are there (1 or 0)
                        if(single_deletions == 0 && double_deletions == 0 && trimer_deletions == 0)
                        {
                            matches_for_all_X++;
                            X_match++;
                        }
                     }
                 }
                 int col_found_aminos = i+1;
                 int row_found_aminos = j+4;
                 virsheet.add_cell(col_found_aminos, row_found_aminos, String.valueOf(amino_match), FORMAT_FOR_TABLE, true);
                 virsheet.add_cell(col_found_aminos, row_found_aminos+1, String.valueOf(X_match), FORMAT_FOR_TABLE, true);
           
                 int col_dels = i+1;
                 virsheet.add_cell(col_dels, ROW_TRI_DELS, String.valueOf(trimer_deletions), FORMAT_FOR_TABLE, true);
                 virsheet.add_cell(col_dels, ROW_SINGLE_DELS, String.valueOf(single_deletions), FORMAT_FOR_TABLE, true);
                 virsheet.add_cell(col_dels, ROW_DOUB_DELS, String.valueOf(double_deletions), FORMAT_FOR_TABLE, true);
             }
             total_single_deletions += single_deletions; //Add the current number of single dels to the total list
             total_double_deletions += double_deletions; //Add the current number of double dels to the total list
             total_trimer_deletions += trimer_deletions; //Add the current number of trimer dels to the total list
          
             virsheet.add_cell(COL_TOTAL, ROW_TRI_DELS,String.valueOf(total_trimer_deletions/number_of_aminos), FORMAT_FOR_TABLE, true);
             virsheet.add_cell(COL_TOTAL, ROW_SINGLE_DELS,String.valueOf(total_single_deletions/number_of_aminos), FORMAT_FOR_TABLE, true);
             virsheet.add_cell(COL_TOTAL, ROW_DOUB_DELS,String.valueOf(total_double_deletions/number_of_aminos), FORMAT_FOR_TABLE, true); 
             virsheet.add_cell(COL_TOTAL ,ROW_SUBS,String.valueOf(all_substitutions), FORMAT_FOR_TABLE, true);
             virsheet.add_cell(COL_TOTAL, ROW_SUBS-1, String.valueOf(matches_for_all_X/number_of_aminos), FORMAT_FOR_TABLE, true);
         }
                            
         for(int z=0; z < number_of_aminos; z++) //Write how often a amino had a match in the total row
         {
             int row_total = z+4;
             virsheet.add_cell(COL_TOTAL, row_total, String.valueOf(all_matches_from_a_amino[z]), FORMAT_FOR_TABLE, true);
         }
                               
        for(int h=0; h < num_of_mut_pos; h++) //Write how many substitutions was at a mut. pos.
        {
             int col_subs = h+1;
             virsheet.add_cell(col_subs, ROW_SUBS, String.valueOf(subs_at_mut_pos[h]), FORMAT_FOR_TABLE, true);
        }
        try {
            virsheet.write_to_excel_sheet(sheet, MAX_ROW, MAX_COL, false);
        } catch (WriteException ex) {
            GUI.addWarningMessage("Error while writing the virtual_sheet - amino acid distribution is not included");
        }
    }
    
    /**Creates Table 6
     * 
     * @param moities The used moitiesfile
     * @param workbook The file where the table is written in
     */
    public static void deletions_and_changes(Moities moities, WritableWorkbook workbook, java.lang.Boolean use_mutsides,
            String single_deletion, String double_deletion, String trimer_deletion, Virtual_Sheet virsheet)
    {
        /*Makes a new sheet named "Table 6"*/
        final int SHEETNUMBER = 4;
        WritableSheet sheet = workbook.createSheet("Deletions and changes", SHEETNUMBER);  
        withoutExceptions = true;
        
        final String STR_TITLE = "Number of deletions, transitions, and transversions";
        final String STR_DEL_IN_CODONS = "Number of deletions in a codon";
        final String STR_SINGLE_NUCL = "Single nucleotide";
        final String STR_TWO_DELS = "Two adjacent nucleotides";
        final String STR_THREE_DELS = "Three adjacent nucleotides";
        final String STR_TRANSITIONS_OUT = "Number of transitions outside the mutagenized codons";
        final String STR_TRANSITIONS_IN = "Number of transitions in all codons";
        final String STR_TRANSVERSIONS_OUT = "Number of transversions outside the mutagenized codons";
        final String STR_TRANSVERSIONS_IN = "Number of transversions in all codons";
        final String STR_REFERENCE = "Reference";
        final String STR_SUBST = "Substitution";
        final String STR_TOTAL = "Total";
        
        final int COL_TITLE = 1;
        final int ROW_TITLE = 0;
        final int COL_SUBTITLE_IN = 0;
        final int COL_SUBTITLE = COL_SUBTITLE_IN+4;
        final int ROW_DEL_IN_CODONS = 2;
        final int COL_SINGLE_NUCL = 0;
        final int ROW_SINGLE_NUCL = 3;
        final int COL_TWO_DELS = 0;
        final int ROW_TWO_DELS = 4;
        final int COL_THREE_DELS = 0;
        final int ROW_THREE_DELS = 5;
        final int ROW_TRANSITIONS = 7;
        final int COL_REFERENCE_IN = 0;
        final int COL_REFERENCE = COL_REFERENCE_IN+4;
        final int ROW_TRANSITIONS_ROW = 8;
        final int COL_TOTAL_IN = 2;
        final int COL_TOTAL = COL_TOTAL_IN+4;
        final int COL_SUBST_IN = 1;
        final int COL_SUBST = COL_SUBST_IN+4;
        
        virsheet.add_cell(COL_SINGLE_NUCL, ROW_SINGLE_NUCL, STR_SINGLE_NUCL, FORMAT_FOR_TABLE, false, 15);
        virsheet.add_cell(COL_TWO_DELS, ROW_TWO_DELS, STR_TWO_DELS, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_THREE_DELS, ROW_THREE_DELS, STR_THREE_DELS, FORMAT_FOR_TABLE, false, 20);
        virsheet.add_cell(COL_TITLE, ROW_TITLE, STR_TITLE, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SUBTITLE_IN, ROW_DEL_IN_CODONS, STR_DEL_IN_CODONS, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SINGLE_NUCL, ROW_SINGLE_NUCL, STR_SINGLE_NUCL, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_TWO_DELS, ROW_TWO_DELS, STR_TWO_DELS, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_THREE_DELS, ROW_THREE_DELS, STR_THREE_DELS, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SUBTITLE, ROW_TRANSITIONS, STR_TRANSITIONS_OUT, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SUBTITLE_IN, ROW_TRANSITIONS, STR_TRANSITIONS_IN, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_REFERENCE, ROW_TRANSITIONS_ROW, STR_REFERENCE, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_REFERENCE_IN, ROW_TRANSITIONS_ROW, STR_REFERENCE, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SUBST, ROW_TRANSITIONS_ROW, STR_SUBST, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_SUBST_IN, ROW_TRANSITIONS_ROW, STR_SUBST, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_TOTAL, ROW_TRANSITIONS_ROW, STR_TOTAL, FORMAT_FOR_TABLE, false);
        virsheet.add_cell(COL_TOTAL_IN, ROW_TRANSITIONS_ROW, STR_TOTAL, FORMAT_FOR_TABLE, false);
   
        //TODO: Auslagern??
        Alignment alignment = moities.get_alignment(); //Get the alignment to get the number of nucleotides
        int num_nucleo = alignment.number_of_nucleotides(); //Get the number of nucleotides
        int num_all_sequences = alignment.number_of_all_sequences(); //Get the number of all sequences
        
        /*Counters for the different kinds of deletions**/
        int single_dels = 0;
        int double_dels = 0;
        int trimer_dels = 0;
        int ind_wildtype = use_mutsides ? 1 : 0;
        
        for(int n=ind_wildtype + 1; n<num_all_sequences;n++) //Go through all sequences (without mut. and wildtype)
        {    
            for(int m=0; m<num_nucleo;m += 3) //Go through all nucleotides in triplet-steps (+3)
            {
                if(m < num_nucleo && m+1 < num_nucleo && m+2 < num_nucleo) 
                {
                    Triplet triplet = moities.get_triplet(n, m); //Get the current triplet to compare it with the wildtype
                    Triplet wildtype = moities.get_triplet(ind_wildtype, m); //Get the wildtype-triplet
                    Amino_Acid amino = new Amino_Acid(triplet); //Make a amino acid out of the current triplet
                    char amino_char = amino.to_char(); //Get the char of the current amino
                    Quintuple<String, Integer, Integer,Integer, Integer> char_and_deletions = null;
                  
                    if(triplet != wildtype) //If the two chars are not the same
                    {
                        char_and_deletions = compare_char(triplet, amino_char,
                                single_deletion, double_deletion, trimer_deletion); //Compare the chars
                        single_dels += Tuple.get2(char_and_deletions); //Get if there is a single deletion
                        double_dels += Tuple.get3(char_and_deletions); //Get if there is a double deletion
                        trimer_dels += Tuple.get4(char_and_deletions); //Get if there is a trimer deletion
                    }
                }
            }
        }
        final int COL_SINGLE_DEL = COL_SINGLE_NUCL + 1;
        final int COL_DOUBLE_DEL = COL_TWO_DELS +1;
        final int COL_TRIMER_DEL = COL_THREE_DELS +1;
        
        virsheet.add_cell(COL_SINGLE_DEL,ROW_SINGLE_NUCL,String.valueOf(single_dels), FORMAT_FOR_TABLE, true);
        virsheet.add_cell(COL_DOUBLE_DEL,ROW_TWO_DELS,String.valueOf(double_dels), FORMAT_FOR_TABLE, true);
        virsheet.add_cell(COL_TRIMER_DEL,ROW_THREE_DELS,String.valueOf(trimer_dels), FORMAT_FOR_TABLE, true);
  
        Map<Pair<Character,Character>, Integer> char_changes = new HashMap<Pair<Character,Character>, Integer>();
        Pair<Character,Character> different_chars = null;
        
        for(int l=ind_wildtype+1;l<num_all_sequences;l++) //Go through all sequences (without mut. and wildtype)
        {
            for(int t=0; t<num_nucleo; t++) //Go trough all nucleotides
            {
                char reference = alignment.get_character(0, t); //If there is a 'N' the transversions shouldn't be count
                char wildtype = alignment.get_character(ind_wildtype, t); //Get the wildtype base
                char current_char = alignment.get_character(l, t); //Get the current base
                    
                if(reference != 'N' && wildtype != current_char && current_char != '-' && current_char != 'N')
                //If the char from the mut. seq. is no N and the wildytype and current char differ and there is no gap
                {
                    different_chars = Pair.from(wildtype, current_char); //Make a pair out of the substitution
                    if(char_changes.get(different_chars) != null) //If there is already an number saved which has to increase
                    {
                        int number_subst = char_changes.get(different_chars); //Get the number...
                        number_subst += 1; //...and add one to it (because it was one more hit found)
                        char_changes.put(different_chars, number_subst);   //then save the new number
                    }
                    else 
                        char_changes.put(different_chars, 1); //If this substitutions wasn't found till now set a one
                 }
             }   
         }
         int l=1; //Counter to change the row correct
         char[] legal_chars = null;
         try {
             legal_chars = Alphabet.return_legal_characters(); //Get all legal chars
        
             for(int i=0;i<legal_chars.length;i++) //Go through all legal chars
             {
                 for(int j=0; j<legal_chars.length;j++) //Go through all legal chars
                 {
                     char first_char = legal_chars[i]; //Get the char to compare with
                     char second_char = legal_chars[j]; //Get the char to which it is compared
                
                     if(Alphabet.is_same_group_change(first_char, second_char) //If there is a transition ...
                       && first_char != '-' && second_char != '-'
                       && first_char != 'N' && second_char != 'N') //... and none of the chars is a deletion or N
                     {
                         String str_first_char = String.valueOf(first_char); //Make a string from the char to write it later
                         String str_second_char = String.valueOf(second_char); //Make a string from the char for writing
                         Pair<Character, Character> pair_chars = Pair.from(first_char, second_char); //Make a tuple pair to get it from the map
                         int num_transitions = 0; //Reset the number of transitions so that if there is no entry for this substitution on the map there can be written a zero
                         if(char_changes.get(pair_chars) != null) //If there is a entry for this substitution
                         {
                             num_transitions = char_changes.get(pair_chars); //Get the number how often this substitution was found
                         }
                         virsheet.add_cell(COL_REFERENCE,ROW_TRANSITIONS_ROW+l, str_first_char, FORMAT_FOR_TABLE, false);
                         virsheet.add_cell(COL_SUBST,ROW_TRANSITIONS_ROW+l, str_second_char, FORMAT_FOR_TABLE, false);
                         virsheet.add_cell(COL_TOTAL, ROW_TRANSITIONS_ROW + l, String.valueOf(num_transitions), FORMAT_FOR_TABLE, true);
                         ++l; //Add one so the next hit will be written one row below the other
                     }
                 }
             }
         } catch (Exception ex) {
             GUI.addWarningMessage("Unknown exception occured: " + ex);
             withoutExceptions = false;
             workbook.removeSheet(SHEETNUMBER);
         }    
    
        Map<Pair<Character,Character>, Integer> char_changes2 = new HashMap<Pair<Character,Character>, Integer>();
        Pair<Character,Character> different_chars2 = null;
        
        for(int m=ind_wildtype+1;m<num_all_sequences;m++) //Go through all sequences (without mut. and wildtype)
        {
            for(int t=0; t<num_nucleo; t++) //Go trough all nucleotides
            {
                char wildtype = alignment.get_character(ind_wildtype, t); //Get the wildtype base
                char current_char = alignment.get_character(m, t); //Get the current base
                    
                if(wildtype != current_char && current_char != '-' && current_char != 'N')
                //If the char from the mut. seq. is no N and the wildytype and current char differ and there is no gap
                {
                    different_chars2 = Pair.from(wildtype, current_char); //Make a pair out of the substitution
                    if(char_changes2.get(different_chars2) != null) //If there is already an number saved which has to increase
                    {
                        int number_subst2 = char_changes2.get(different_chars2); //Get the number...
                        number_subst2 += 1; //...and add one to it (because it was one more hit found)
                        char_changes2.put(different_chars2, number_subst2);   //then save the new number
                    }
                    else 
                        char_changes2.put(different_chars2, 1); //If this substitutions wasn't found till now set a one
                 }
             }
         }
        
         int m=1; //Counter to change the row correct
         char[] legal_chars2 = null;
         try {
             legal_chars2 = Alphabet.return_legal_characters(); //Get all legal chars
        
             for(int i=0;i<legal_chars2.length;i++) //Go through all legal chars
             {
                 for(int j=0; j<legal_chars2.length;j++) //Go through all legal chars
                 {
                     char first_char = legal_chars2[i]; //Get the char to compare with
                     char second_char = legal_chars2[j]; //Get the char to which it is compared
                
                     if(Alphabet.is_same_group_change(first_char, second_char) //If there is a transition ...
                       && first_char != '-' && second_char != '-'
                       && first_char != 'N' && second_char != 'N') //... and none of the chars is a deletion or N
                     {
                         String str_first_char = String.valueOf(first_char); //Make a string from the char to write it later
                         String str_second_char = String.valueOf(second_char); //Make a string from the char for writing
                         Pair<Character, Character> pair_chars = Pair.from(first_char, second_char); //Make a tuple pair to get it from the map
                         int num_transitions = 0; //Reset the number of transitions so that if there is no entry for this substitution on the map there can be written a zero
                         if(char_changes2.get(pair_chars) != null) //If there is a entry for this substitution
                         {
                             num_transitions = char_changes2.get(pair_chars); //Get the number how often this substitution was found
                         }
                         virsheet.add_cell(COL_REFERENCE_IN,ROW_TRANSITIONS_ROW+m, str_first_char, FORMAT_FOR_TABLE, false);
                         virsheet.add_cell(COL_SUBST_IN,ROW_TRANSITIONS_ROW+m, str_second_char, FORMAT_FOR_TABLE, false);
                         virsheet.add_cell(COL_TOTAL_IN, ROW_TRANSITIONS_ROW + m, String.valueOf(num_transitions), FORMAT_FOR_TABLE, true);
                         ++m; //Add one so the next hit will be written one row below the other
                     }
                 }
             }
         } catch (Exception ex) {
             GUI.addWarningMessage("Unknown exception occured: " + ex);
             withoutExceptions = false;
             workbook.removeSheet(SHEETNUMBER);
         }
         if(withoutExceptions) {
             final int ROW_TRANSVERSIONS = ROW_TRANSITIONS_ROW + l + 1;
             final int ROW_TRANSVERSIONS_ROW = ROW_TRANSVERSIONS +1;
        
             virsheet.add_cell(COL_SUBTITLE, ROW_TRANSVERSIONS, STR_TRANSVERSIONS_OUT, FORMAT_FOR_TABLE, false);
             virsheet.add_cell(COL_SUBTITLE_IN, ROW_TRANSVERSIONS, STR_TRANSVERSIONS_IN, FORMAT_FOR_TABLE, false);                          
             virsheet.add_cell(COL_REFERENCE, ROW_TRANSVERSIONS_ROW, STR_REFERENCE, FORMAT_FOR_TABLE, false);
             virsheet.add_cell(COL_REFERENCE_IN, ROW_TRANSVERSIONS_ROW, STR_REFERENCE, FORMAT_FOR_TABLE, false);
             virsheet.add_cell(COL_SUBST, ROW_TRANSVERSIONS_ROW, STR_SUBST, FORMAT_FOR_TABLE, false);
             virsheet.add_cell(COL_SUBST_IN, ROW_TRANSVERSIONS_ROW, STR_SUBST, FORMAT_FOR_TABLE, false);
             virsheet.add_cell(COL_TOTAL, ROW_TRANSVERSIONS_ROW, STR_TOTAL, FORMAT_FOR_TABLE, false);
             virsheet.add_cell(COL_TOTAL_IN, ROW_TRANSVERSIONS_ROW, STR_TOTAL, FORMAT_FOR_TABLE, false);
             
             int k=1; //Counter to change the row correct
             for(int i=0;i<legal_chars.length;i++) //Go through all legal chars
             {
                 for(int j=0; j<legal_chars.length;j++) //Go through all legal chars
                 {
                     char first_char = legal_chars[i]; //Get the char to compare with
                     char second_char = legal_chars[j]; //Get the char to which it is compared
                     try {
                         if(Alphabet.is_different_group_change(first_char, second_char) //If there is a transversion
                            && first_char != '-' && second_char != '-'
                            && first_char != 'N' && second_char != 'N') //and none of the chars is a deletion or N
                         {
                             String str_first_char = String.valueOf(first_char); //Make a string from the char to write it later
                             String str_second_char = String.valueOf(second_char); //Make a string from the char for writing
                             Pair<Character, Character> pair_chars = Pair.from(first_char, second_char); //Make a tuple pair to get it from the map
                             int num_transversions = 0; //Reset the number of transitions so that if there is no entry for this substitution on the map there can be written a zero
                             if(char_changes.get(pair_chars) != null) //If there is a entry for this substitution
                             {
                                 num_transversions = char_changes.get(pair_chars); //Get the number how often this substitution was found
                             }   
                                 int row_transversions = ROW_TRANSVERSIONS_ROW+k;
                                 virsheet.add_cell(COL_REFERENCE,row_transversions, str_first_char, FORMAT_FOR_TABLE, false);
                                 virsheet.add_cell(COL_SUBST,row_transversions, str_second_char, FORMAT_FOR_TABLE, false);
                                 virsheet.add_cell(COL_TOTAL,row_transversions, String.valueOf(num_transversions), FORMAT_FOR_TABLE, true);
                                 ++k;
                         }
                     } catch (Exception ex) {
                         GUI.addWarningMessage("Unknown exception occured: " + ex);
                         withoutExceptions = false;
                         workbook.removeSheet(SHEETNUMBER);
                     }  
                }
            }
             
            k=1; //Counter to change the row correct
             for(int i=0;i<legal_chars.length;i++) //Go through all legal chars
             {
                 for(int j=0; j<legal_chars.length;j++) //Go through all legal chars
                 {
                     char first_char = legal_chars[i]; //Get the char to compare with
                     char second_char = legal_chars[j]; //Get the char to which it is compared
                     try {
                         if(Alphabet.is_different_group_change(first_char, second_char) //If there is a transversion
                            && first_char != '-' && second_char != '-'
                            && first_char != 'N' && second_char != 'N') //and none of the chars is a deletion or N
                         {
                             String str_first_char = String.valueOf(first_char); //Make a string from the char to write it later
                             String str_second_char = String.valueOf(second_char); //Make a string from the char for writing
                             Pair<Character, Character> pair_chars = Pair.from(first_char, second_char); //Make a tuple pair to get it from the map
                             int num_transversions = 0; //Reset the number of transitions so that if there is no entry for this substitution on the map there can be written a zero
                             if(char_changes2.get(pair_chars) != null) //If there is a entry for this substitution
                             {
                                 num_transversions = char_changes2.get(pair_chars); //Get the number how often this substitution was found
                             }   
                                 int row_transversions = ROW_TRANSVERSIONS_ROW+k;
                                 virsheet.add_cell(COL_REFERENCE_IN,row_transversions, str_first_char, FORMAT_FOR_TABLE, false);
                                 virsheet.add_cell(COL_SUBST_IN,row_transversions, str_second_char, FORMAT_FOR_TABLE, false);
                                 virsheet.add_cell(COL_TOTAL_IN,row_transversions, String.valueOf(num_transversions), FORMAT_FOR_TABLE, true);
                                 ++k;
                         }
                     } catch (Exception ex) {
                         GUI.addWarningMessage("Unknown exception occured: " + ex);
                         withoutExceptions = false;
                         workbook.removeSheet(SHEETNUMBER);
                     }  
                }           
            }
             
            if(withoutExceptions) {
            final int ROW_NUM_SEQ = ROW_TRANSVERSIONS_ROW + k + 1;
            final int COL_NUMBER_SEQ = COL_SUBTITLE_IN + 1;
            final String STR_NUM_SEQ = "Number of sequences";
            int num_seq = alignment.number_of_all_sequences() - (ind_wildtype + 1); //Without reference and wildtype
            if(use_mutsides) {
                virsheet.add_cell(COL_SUBTITLE_IN, ROW_NUM_SEQ,STR_NUM_SEQ, FORMAT_FOR_TABLE, false);

            } else {
                virsheet.add_cell(COL_SUBTITLE_IN, ROW_NUM_SEQ,STR_NUM_SEQ, FORMAT_FOR_TABLE, false);

            }
            virsheet.add_cell(COL_SUBTITLE_IN, ROW_NUM_SEQ,STR_NUM_SEQ, FORMAT_FOR_TABLE, false);
            virsheet.add_cell(COL_NUMBER_SEQ, ROW_NUM_SEQ, String.valueOf(num_seq), FORMAT_FOR_TABLE, true); 
            }
        }
         
         
         
        try {
            virsheet.write_to_excel_sheet(sheet, MAX_ROW, MAX_COL, true);
        } catch (WriteException ex) {
            GUI.addWarningMessage("Error while writing the virtual_sheet - deletions and changes is not included");
        }
    }
    
    /**Counts the deletions
     * 
     * @param alignment The used alignment
     * @param filename ?
     * 
     * @return Number of found deletions
     */
    public static int[][] number_of_deletions(Alignment alignment, String filename, java.lang.Boolean use_mutsides)
    {
         Moities moities = new Moities(alignment, use_mutsides);
         int num_of_mut_pos = moities.get_num_mutagenized_positions(use_mutsides); //Gets the Number of mutagenized positions
         int[] mut_pos = moities.get_mutagenized_positions(num_of_mut_pos, use_mutsides); //Get the mut.pos
        
        int trimer_deletions_elsewhere = 0;
        int double_deletions_elsewhere = 0;
        int single_deletions_elsewhere = 0;
        
        boolean current_is_mut_pos = false;

        int num_sequences = alignment.number_of_all_sequences();
        int num_nucleos = alignment.number_of_nucleotides();
        
        int[][] number_deletions = new int[num_sequences][3];
        int ind_wildtype = use_mutsides ? 1 : 0;

        for(int i=ind_wildtype+1; i < num_sequences;i++) //Go through all entries/sequences
        {
            for(int j=0; j < num_nucleos;j+=3) //Go through all nucleotides
            {
                int k=0; //Resest for the next test
                while(k < num_of_mut_pos && current_is_mut_pos != true)
                { 
                    if(j != mut_pos[k]) //Checks if the current position is none of the mutagenzied
                    {   
                        k++; //Check for the next position
                        current_is_mut_pos = false; //Save: didn't find a postion
                    }
                    else //if j is a mut. position and the boolean isn't true yet
                        current_is_mut_pos = true; //Save: The current postion is a mut. one
                }
                    if(current_is_mut_pos == false && j<num_nucleos &&
                        j+1<num_nucleos && j+2<num_nucleos) //If the current position is not one of the mutagenzied
                    {
                        char triplet_char_1 = alignment.get_character(i, j); //Get first triplet-char
                        char triplet_char_2 = alignment.get_character(i, j+1); //Get second triplet-char
                        char triplet_char_3 = alignment.get_character(i, j+2); //Get third triplet-char
                
                        String triplet_str = String.valueOf(triplet_char_1) 
                            + String.valueOf(triplet_char_2) + String.valueOf(triplet_char_3); //makes a string out of the chars

                        if(java.util.regex.Pattern.compile("---").matcher(triplet_str).find())
                            //If there are three deletions following each other
                        {
                           trimer_deletions_elsewhere++; //Notice that there is a trimer-del
                        }
                        else if(java.util.regex.Pattern.compile("--").matcher(triplet_str).find())
                            //If there are two deletions
                        {
                            double_deletions_elsewhere++; //Notice that there is a double-del
                        }
                        else if(java.util.regex.Pattern.compile("-").matcher(triplet_str).find())
                            //If there is at least one single deletion
                        {
                            if(triplet_char_1 == '-') //If the deletion is in the first tripletcharacter
                            {
                                single_deletions_elsewhere++; //Notice that there is a single-del
                            }
                            if(triplet_char_2 == '-') //If the del is in the second char
                            {
                                single_deletions_elsewhere++; //Notice that there is a single-del (or a second one)
                            }
                            if(triplet_char_3 == '-') //If the del is at the third char
                            {
                                single_deletions_elsewhere++; //Notice that there is one more single-del
                            }
                        }
                  } 
                  current_is_mut_pos = false; //Reset for next check
            }
            number_deletions[i][0] = single_deletions_elsewhere;
            number_deletions[i][1] = double_deletions_elsewhere;
            number_deletions[i][2] = trimer_deletions_elsewhere;
            //number_deletions[Number of Sequence][Charnumber]
            
            single_deletions_elsewhere = 0; //Reset for a new loop
            double_deletions_elsewhere = 0; //Reset for a new loop
            trimer_deletions_elsewhere = 0; //Reset for a new loop
        }
        return number_deletions; //How much deletions was found (as an array)
    }

    /**Compares two chars. 
     * If they are different the amino_char will be written. 
     * If there is a 'X' there will be a check if there is a deletion or a not known amino acid.
     * Can identify d1, d1d1, d2 and d3.
     * 
     * @param triplet The Triplet which should be checked
     * @param amino_char The amino_char that differs
     * @return If there is a deletion (d1, d1d1, d2, d3), an unknown amino acid ('X')
     * or a amino that differs (amino_char). Returntype: Quintuple<String amino_char, int d1, int d1d1, int d2, int d3>
     */
    public static Quintuple<String, Integer, Integer, Integer, Integer> compare_char(Triplet triplet,  char amino_char,
            String single_deletion, String double_deletion, String trimer_deletion)
    {
        String amino_output = "";
        int trimer_deletions=0, double_deletions=0, single_deletions=0, num_subs=0;
           
        if(amino_char == 'X') //If there is no legal amino acid
        {
           String del_in_triplet = triplet.toString(); /*Will get the triplet-string if there is a deletion in the triplet
            * convert the triplet to a string so pattern and matcher can be used*/
                        
           if(java.util.regex.Pattern.compile("---").matcher(del_in_triplet).find()) //If you find a --- (trimer del.)
           {
              ++trimer_deletions; //Found one more trimer del.
              amino_output = trimer_deletion; //So there will be written a d3 in the table
              }
              else if(java.util.regex.Pattern.compile("--").matcher(del_in_triplet).find())
              {
                 ++double_deletions;
                 amino_output = double_deletion; //So there will be written a d2 in the table
              }
              else if(java.util.regex.Pattern.compile("-").matcher(del_in_triplet).find())
              {
                  char[] char_triplet_del = del_in_triplet.toCharArray(); /*Will transform the triplet-string 
          * in a char-array if there is a deletion to check the single positions*/
                  if(char_triplet_del[0] == '-')
                  {
                     ++single_deletions;
                     amino_output = single_deletion; //So there will be written a d1 in the table
                  }
                  if(char_triplet_del[1] == '-')
                  {
                     ++single_deletions;
                     amino_output = amino_output + single_deletion; //So there will be written a d1 in the table
                  }
                  if(char_triplet_del[2] == '-')
                  {
                      ++single_deletions;
                      amino_output = amino_output + single_deletion; /*So there will be written a d1 in the table 
                      * or if the first pos. was a d1 it will be written d1d1*/
                  }
              }
                    
              else
                 amino_output = "X"; //If there was no deletion the amino acid is illegal
          }
          else
          {
             amino_output = String.valueOf(amino_char);
             ++num_subs;
          }
       return Quintuple.from(amino_output, single_deletions, double_deletions, trimer_deletions, num_subs);
    }

}
