package bibalign;

/**This class represents a triplet of nucleotides
 * 
 * @author Daniela Buchwald
 *
 */
public class Triplet {
	
	private char[] triplet = new char[3];
	
	/**Construct triplet from string 
	 * (has to have length 3 and composed 
	 * of legal characters)
	 * 
	 * @param str The string which represents the triplet
         * @throws IllegalArgumentException
	 */
	public Triplet(String str) 
                throws IllegalArgumentException
        {
            char[] legal_chars = null;
            char[] triplet_chars = str.toCharArray();
            try {
                legal_chars = Alphabet.return_legal_characters();
            } catch (Exception ex) {
                GUI.addWarningMessage("Unknows Exception occured: " + ex);
            }
            int length_triplet_chars = triplet_chars.length;
            int number_of_legal_chars = legal_chars.length;
            
            boolean first_char=false, second_char=false, third_char=false;
            
            if(length_triplet_chars == 3)
            {
                for(int i=0; i<number_of_legal_chars;i++)
                {
                    if(triplet_chars[0] == legal_chars[i])
                        first_char = true;
                    if(triplet_chars[1] == legal_chars[i])
                        second_char = true;
                    if(triplet_chars[2] == legal_chars[i])
                        third_char = true;
                }
            
                if(first_char == true && second_char == true && third_char == true) //Only if the triplet contains 3 chars which are legal
                    triplet = triplet_chars; //Converts the given string to a char-array
                else
                {
                    if(first_char == false)
                        throw new IllegalArgumentException("Error: Unlegal tripletchar 1: " + triplet_chars[0]);
                    if(second_char == false)
                        throw new IllegalArgumentException("Error: Unlegal tripletchar 2: " + triplet_chars[1]);
                    if(third_char == false)
                        throw new IllegalArgumentException("Error: Unlegal tripletchar 3: " + triplet_chars[2]);
                }
            }
            else
                throw new IllegalArgumentException("Error: Triplet to long/short");
        }
        
        /**Gets the representation of a triplet as string
         * 
         * @return The triplet as a string
         */
       @Override
        public String toString()
       {
               String triplet_string = String.valueOf(triplet[0]) + String.valueOf(triplet[1])
                + String.valueOf(triplet[2]);
                    return triplet_string;
       }
        
 
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Triplet other = (Triplet) obj;
        if (!toString().equals(other.toString ()))
            return false;
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = toString().hashCode();
        return hash;
    }
}
