package bibalign;

import java.util.*;
import com.mallardsoft.tuple.*;
import jxl.*; 
import jxl.read.biff.*;
import java.io.*;

/**the amino acids in use and the relation between 
 * the triplets and the amino acids (Table 1)
 * 
 * @author Daniela Buchwald
 *
 */
public class Used_Amino_Acids {
	
	 private Map<Triplet, Amino_Acid> triplet_to_amino_acid = new HashMap<Triplet, Amino_Acid>();
         private static ArrayList<Character> aminoacids = null;
         
	/**Construct from file (Table 1)
	 *  
	 * @param filename
	 * @throws IllegalArgumentException
	 */
	public Used_Amino_Acids(String filename)
                throws IllegalArgumentException
	{
            ArrayList<Character> aminos = new ArrayList<Character>();

            try {
                Workbook workbook = Workbook.getWorkbook(new java.io.File(filename));
                Sheet table1 = workbook.getSheet(0);
                int number_aminos = table1.getRows() - 1; //Number of Rows in the file (Number of Aminoacids + Title, so -1)
            
                for(int i=1; i<=number_aminos; i++) //i = line
                {
                    String triplet_str = table1.getCell(0, i).getContents().toString(); //Get triplets
                    String amino_str = table1.getCell(1, i).getContents().toString(); //Get aminos
                    char amino_char = amino_str.charAt(0);
                    if(amino_str.length() == 1) {
                        if(!aminos.contains(amino_char))
                            aminos.add(amino_char);
                    }
                    else
                    {
                            throw new IllegalArgumentException("Illegal amino acids - longer than one character");
                    }
                    Triplet triplet = null;
                    Boolean triplet_is_legal = false;
                    try {
                        triplet = new Triplet(triplet_str);
                        triplet_is_legal = true;
                    } catch(IllegalArgumentException ex) {
                        GUI.addWarningMessage("An Error occured while reading the AminoAcidsTable: " + ex.getMessage() 
                                + "\n Triplet " +triplet_str+ " will not be added to the table");
                        triplet_is_legal = false;
                    }
                    if(triplet_is_legal) {
                        Amino_Acid amino = new Amino_Acid(amino_char);
                        triplet_to_amino_acid.put(triplet, amino);
                    }

                }
                aminoacids = aminos;
            } catch (IOException ex) {
                System.err.println(ex);
                ex.printStackTrace();
                System.exit(-1);
            } catch (BiffException ex) {
                System.err.println(ex);
                ex.printStackTrace();
                System.exit(-1);
            } catch (Exception ex) {
                System.err.println(ex);
                ex.printStackTrace();
                System.exit(-1);
            }
	}
        
	/**Converts a triplet to an amino acid, 
         * also notifying whether the triplet was 
         * legal (i.e. in Table 1)
         * 
         * @param triplet The Triplet from that the amino acid should be returned
         * @return The amino acid which is represented by the given triplet
         */
	public Pair<Character, java.lang.Boolean> convert(Triplet triplet)
	{
            final char illegalAmino = 'X';
           if(triplet_to_amino_acid != null) //If the map is initiated
           {
               Pair<Character, java.lang.Boolean> return_value = null;
                Amino_Acid amino = triplet_to_amino_acid.get(triplet);
                if(amino != null)
                {
                    return_value = Pair.from(amino.to_char(),true);
                }
                else {
                    try {
                    amino = new Amino_Acid(illegalAmino); 
                    return_value = Pair.from(amino.to_char(),false);
                    } catch (Exception ex) {
                        System.err.println(ex);
                        ex.printStackTrace();
                        System.exit(-1);
                    }
                }
            
                    return return_value;
                }
           else //If the map is not initiated
           {
               System.out.println("Map not initialized");
               return null;
           }
        }
        
        /**Returns the list of used aminos
         * 
         * @return The list of all used amino acids
         */
        public static ArrayList<Character> amino_list()
        {
            return aminoacids;
        }

        private ArrayList<Character> sort(ArrayList<Character> aminos) {

            for(int i=0; i < aminos.size()-1; i++) {
                Character amino_one = aminos.get(i);
                

            }

            return aminos;
        }
}
